@@ -35,10 +35,6 @@ if [ ${missing_env} -gt 0 ]; then
 fi
 
 YETUS_ARGS=()
-if [[ -n "${MULTIJDK}" ]]; then
-  YETUS_ARGS=("--multijdktests=compile,javadoc" "${YETUS_ARGS[@]}")
-  YETUS_ARGS=("--multijdkdirs=${MULTIJDK}" "${YETUS_ARGS[@]}")
-fi
 
 # If we're doing docker, make sure we don't accidentally pollute the image with a host java path
 if [ -n "${JAVA_HOME}" ]; then
@@ -69,14 +65,8 @@ YETUS_ARGS=("--sentinel" "${YETUS_ARGS[@]}")
 YETUS_ARGS=("--branch=${BRANCH_NAME}" "${YETUS_ARGS[@]}")
 YETUS_ARGS=("--tests-filter=${TESTS_FILTER}" "${YETUS_ARGS[@]}")
 YETUS_ARGS=("--ignore-unknown-options=true" "${YETUS_ARGS[@]}")
-# Why are these not being picked up from hbase-personality?
-YETUS_ARGS=("--proclimit=10000" "${YETUS_ARGS[@]}")
 YETUS_ARGS=("--dockermemlimit=20g" "${YETUS_ARGS[@]}")
 
-if [[ "$SKIP_ERRORPRONE" = true ]]; then
-  YETUS_ARGS=("--skip-errorprone" "${YETUS_ARGS[@]}")
-fi
-
 if [[ -n "${EXCLUDE_TESTS_URL}" ]]; then
   YETUS_ARGS=("--exclude-tests-url=${EXCLUDE_TESTS_URL}" "${YETUS_ARGS[@]}")
 fi
@@ -86,7 +76,15 @@ fi
 
 # For testing with specific hadoop version. Activates corresponding profile in maven runs.
 if [[ -n "${HADOOP_PROFILE}" ]]; then
-  YETUS_ARGS=("--hadoop-profile=${HADOOP_PROFILE}" "${YETUS_ARGS[@]}")
+  # Master has only Hadoop3 support. We don't need to activate any profile.
+  # The Jenkinsfile should not attempt to run any Hadoop2 tests.
+  if [[ "${BRANCH_NAME}" =~ branch-2* ]]; then
+    YETUS_ARGS=("--hadoop-profile=${HADOOP_PROFILE}" "${YETUS_ARGS[@]}")
+  fi
+fi
+
+if [[ -n "${SKIP_ERROR_PRONE}" ]]; then
+  YETUS_ARGS=("--skip-errorprone" "${YETUS_ARGS[@]}")
 fi
 
 if [[ true == "${DEBUG}" ]]; then
@@ -98,11 +96,6 @@ if [[ ! -d "${OUTPUT_DIR}" ]]; then
   exit 1
 fi
 
-# pass asf nightlies url in
-if [[ -n "${ASF_NIGHTLIES_GENERAL_CHECK_BASE}" ]]; then
-  YETUS_ARGS=("--asf-nightlies-general-check-base=${ASF_NIGHTLIES_GENERAL_CHECK_BASE}" "${YETUS_ARGS[@]}")
-fi
-
 if [[ true !=  "${USE_YETUS_PRERELEASE}" ]]; then
   YETUS_ARGS=("--shelldocs=${WORKSPACE}/yetus-${YETUS_RELEASE}/bin/shelldocs" "${YETUS_ARGS[@]}")
   TESTPATCHBIN="${WORKSPACE}/yetus-${YETUS_RELEASE}/bin/test-patch"
