@@ -17,34 +17,37 @@
  */
 package org.apache.hadoop.hbase.client;
 
-import com.google.protobuf.Descriptors.MethodDescriptor;
-import com.google.protobuf.Message;
-import com.yammer.metrics.core.Counter;
-import com.yammer.metrics.core.Histogram;
-import com.yammer.metrics.core.MetricName;
-import com.yammer.metrics.core.MetricsRegistry;
-import com.yammer.metrics.core.Timer;
-import com.yammer.metrics.reporting.JmxReporter;
-import com.yammer.metrics.util.RatioGauge;
-import org.apache.hadoop.hbase.ServerName;
-import org.apache.hadoop.hbase.classification.InterfaceAudience;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos.ClientService;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos.MutateRequest;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos.MutationProto.MutationType;
-import org.apache.hadoop.hbase.util.Bytes;
+import static com.codahale.metrics.MetricRegistry.name;
+import static org.apache.hadoop.hbase.util.ConcurrentMapUtils.computeIfAbsent;
+
+import com.codahale.metrics.Counter;
+import com.codahale.metrics.Histogram;
+import com.codahale.metrics.JmxReporter;
+import com.codahale.metrics.MetricRegistry;
+import com.codahale.metrics.RatioGauge;
+import com.codahale.metrics.Timer;
+import org.apache.hbase.thirdparty.com.google.common.annotations.VisibleForTesting;
 
 import java.util.concurrent.ConcurrentHashMap;
-import java.util.concurrent.ConcurrentSkipListMap;
 import java.util.concurrent.ConcurrentMap;
+import java.util.concurrent.ConcurrentSkipListMap;
 import java.util.concurrent.ThreadPoolExecutor;
 import java.util.concurrent.TimeUnit;
+import java.util.function.Supplier;
+import org.apache.hadoop.hbase.ServerName;
+import org.apache.yetus.audience.InterfaceAudience;
+import org.apache.hbase.thirdparty.com.google.protobuf.Descriptors.MethodDescriptor;
+import org.apache.hbase.thirdparty.com.google.protobuf.Message;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.ClientService;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.MutateRequest;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.MutationProto.MutationType;
+import org.apache.hadoop.hbase.util.Bytes;
 
 /**
  * This class is for maintaining the various connection statistics and publishing them through
  * the metrics interfaces.
  *
- * This class manages its own {@link MetricsRegistry} and {@link JmxReporter} so as to not
+ * This class manages its own {@link MetricRegistry} and {@link JmxReporter} so as to not
  * conflict with other uses of Yammer Metrics within the client application. Instantiating
  * this class implicitly creates and "starts" instances of these classes; be sure to call
  * {@link #shutdown()} to terminate the thread pools they allocate.
@@ -55,7 +58,6 @@ public class MetricsConnection implements StatisticTrackable {
   /** Set this key to {@code true} to enable metrics collection of client requests. */
   public static final String CLIENT_SIDE_METRICS_ENABLED_KEY = "hbase.client.metrics.enable";
 
-  private static final String CNT_BASE = "rpcCount_";
   private static final String DRTN_BASE = "rpcCallDurationMs_";
   private static final String REQ_BASE = "rpcCallRequestSizeBytes_";
   private static final String RESP_BASE = "rpcCallResponseSizeBytes_";
@@ -63,8 +65,6 @@ public class MetricsConnection implements StatisticTrackable {
   private static final String HEAP_BASE = "heapOccupancy_";
   private static final String CACHE_BASE = "cacheDroppingExceptions_";
   private static final String UNKNOWN_EXCEPTION = "UnknownException";
-  private static final String NS_LOOKUPS = "nsLookups";
-  private static final String NS_LOOKUPS_FAILED = "nsLookupsFailed";
   private static final String CLIENT_SVC = ClientService.getDescriptor().getName();
 
   /** A container class for collecting details about the RPC call as it percolates. */
@@ -74,6 +74,7 @@ public class MetricsConnection implements StatisticTrackable {
     private long startTime = 0;
     private long callTimeMs = 0;
     private int concurrentCallsPerServer = 0;
+    private int numActionsPerServer = 0;
 
     public long getRequestSizeBytes() {
       return requestSizeBytes;
@@ -114,26 +115,38 @@ public class MetricsConnection implements StatisticTrackable {
     public void setConcurrentCallsPerServer(int callsPerServer) {
       this.concurrentCallsPerServer = callsPerServer;
     }
+
+    public int getNumActionsPerServer() {
+      return numActionsPerServer;
+    }
+
+    public void setNumActionsPerServer(int numActionsPerServer) {
+      this.numActionsPerServer = numActionsPerServer;
+    }
   }
 
+  @VisibleForTesting
   protected static final class CallTracker {
     private final String name;
-    final Timer callTimer;
-    final Histogram reqHist;
-    final Histogram respHist;
+    @VisibleForTesting final Timer callTimer;
+    @VisibleForTesting final Histogram reqHist;
+    @VisibleForTesting final Histogram respHist;
 
-    private CallTracker(MetricsRegistry registry, String name, String subName, String scope) {
+    private CallTracker(MetricRegistry registry, String name, String subName, String scope) {
       StringBuilder sb = new StringBuilder(CLIENT_SVC).append("_").append(name);
       if (subName != null) {
         sb.append("(").append(subName).append(")");
       }
       this.name = sb.toString();
-      this.callTimer = registry.newTimer(MetricsConnection.class, DRTN_BASE + this.name, scope);
-      this.reqHist = registry.newHistogram(MetricsConnection.class, REQ_BASE + this.name, scope);
-      this.respHist = registry.newHistogram(MetricsConnection.class, RESP_BASE + this.name, scope);
+      this.callTimer = registry.timer(name(MetricsConnection.class,
+        DRTN_BASE + this.name, scope));
+      this.reqHist = registry.histogram(name(MetricsConnection.class,
+        REQ_BASE + this.name, scope));
+      this.respHist = registry.histogram(name(MetricsConnection.class,
+        RESP_BASE + this.name, scope));
     }
 
-    private CallTracker(MetricsRegistry registry, String name, String scope) {
+    private CallTracker(MetricRegistry registry, String name, String scope) {
       this(registry, name, null, scope);
     }
 
@@ -154,29 +167,33 @@ public class MetricsConnection implements StatisticTrackable {
     final Histogram memstoreLoadHist;
     final Histogram heapOccupancyHist;
 
-    public RegionStats(MetricsRegistry registry, String name) {
+    public RegionStats(MetricRegistry registry, String name) {
       this.name = name;
-      this.memstoreLoadHist = registry.newHistogram(MetricsConnection.class,
-          MEMLOAD_BASE + this.name);
-      this.heapOccupancyHist = registry.newHistogram(MetricsConnection.class,
-          HEAP_BASE + this.name);
+      this.memstoreLoadHist = registry.histogram(name(MetricsConnection.class,
+          MEMLOAD_BASE + this.name));
+      this.heapOccupancyHist = registry.histogram(name(MetricsConnection.class,
+          HEAP_BASE + this.name));
     }
 
-    public void update(ClientProtos.RegionLoadStats regionStatistics) {
-      this.memstoreLoadHist.update(regionStatistics.getMemstoreLoad());
+    public void update(RegionLoadStats regionStatistics) {
+      this.memstoreLoadHist.update(regionStatistics.getMemStoreLoad());
       this.heapOccupancyHist.update(regionStatistics.getHeapOccupancy());
     }
   }
 
+  @VisibleForTesting
   protected static class RunnerStats {
     final Counter normalRunners;
     final Counter delayRunners;
     final Histogram delayIntevalHist;
 
-    public RunnerStats(MetricsRegistry registry) {
-      this.normalRunners = registry.newCounter(MetricsConnection.class, "normalRunnersCount");
-      this.delayRunners = registry.newCounter(MetricsConnection.class, "delayRunnersCount");
-      this.delayIntevalHist = registry.newHistogram(MetricsConnection.class, "delayIntervalHist");
+    public RunnerStats(MetricRegistry registry) {
+      this.normalRunners = registry.counter(
+        name(MetricsConnection.class, "normalRunnersCount"));
+      this.delayRunners = registry.counter(
+        name(MetricsConnection.class, "delayRunnersCount"));
+      this.delayIntevalHist = registry.histogram(
+        name(MetricsConnection.class, "delayIntervalHist"));
     }
 
     public void incrNormalRunners() {
@@ -192,8 +209,9 @@ public class MetricsConnection implements StatisticTrackable {
     }
   }
 
+  @VisibleForTesting
   protected ConcurrentHashMap<ServerName, ConcurrentMap<byte[], RegionStats>> serverStats
-          = new ConcurrentHashMap<ServerName, ConcurrentMap<byte[], RegionStats>>();
+          = new ConcurrentHashMap<>();
 
   public void updateServerStats(ServerName serverName, byte[] regionName,
                                 Object r) {
@@ -201,7 +219,7 @@ public class MetricsConnection implements StatisticTrackable {
       return;
     }
     Result result = (Result) r;
-    ClientProtos.RegionLoadStats stats = result.getStats();
+    RegionLoadStats stats = result.getStats();
     if (stats == null) {
       return;
     }
@@ -209,32 +227,15 @@ public class MetricsConnection implements StatisticTrackable {
   }
 
   @Override
-  public void updateRegionStats(ServerName serverName, byte[] regionName,
-    ClientProtos.RegionLoadStats stats) {
+  public void updateRegionStats(ServerName serverName, byte[] regionName, RegionLoadStats stats) {
     String name = serverName.getServerName() + "," + Bytes.toStringBinary(regionName);
-    ConcurrentMap<byte[], RegionStats> rsStats = null;
-    if (serverStats.containsKey(serverName)) {
-      rsStats = serverStats.get(serverName);
-    } else {
-      rsStats = serverStats.putIfAbsent(serverName,
-          new ConcurrentSkipListMap<byte[], RegionStats>(Bytes.BYTES_COMPARATOR));
-      if (rsStats == null) {
-        rsStats = serverStats.get(serverName);
-      }
-    }
-    RegionStats regionStats = null;
-    if (rsStats.containsKey(regionName)) {
-      regionStats = rsStats.get(regionName);
-    } else {
-      regionStats = rsStats.putIfAbsent(regionName, new RegionStats(this.registry, name));
-      if (regionStats == null) {
-        regionStats = rsStats.get(regionName);
-      }
-    }
+    ConcurrentMap<byte[], RegionStats> rsStats = computeIfAbsent(serverStats, serverName,
+      () -> new ConcurrentSkipListMap<>(Bytes.BYTES_COMPARATOR));
+    RegionStats regionStats =
+        computeIfAbsent(rsStats, regionName, () -> new RegionStats(this.registry, name));
     regionStats.update(stats);
   }
 
-
   /** A lambda for dispatching to the appropriate metric factory method */
   private static interface NewMetric<T> {
     T newMetric(Class<?> clazz, String name, String scope);
@@ -245,114 +246,98 @@ public class MetricsConnection implements StatisticTrackable {
   /** Default load factor from {@link java.util.HashMap#DEFAULT_LOAD_FACTOR} */
   private static final float LOAD_FACTOR = 0.75f;
   /**
-   * Anticipated number of concurrent accessor threads, from
-   * {@link ConnectionManager.HConnectionImplementation#getBatchPool()}
+   * Anticipated number of concurrent accessor threads
    */
   private static final int CONCURRENCY_LEVEL = 256;
 
-  private final MetricsRegistry registry;
+  private final MetricRegistry registry;
   private final JmxReporter reporter;
   private final String scope;
 
   private final NewMetric<Timer> timerFactory = new NewMetric<Timer>() {
     @Override public Timer newMetric(Class<?> clazz, String name, String scope) {
-      return registry.newTimer(clazz, name, scope);
+      return registry.timer(name(clazz, name, scope));
     }
   };
 
   private final NewMetric<Histogram> histogramFactory = new NewMetric<Histogram>() {
     @Override public Histogram newMetric(Class<?> clazz, String name, String scope) {
-      return registry.newHistogram(clazz, name, scope);
+      return registry.histogram(name(clazz, name, scope));
     }
   };
 
   private final NewMetric<Counter> counterFactory = new NewMetric<Counter>() {
     @Override public Counter newMetric(Class<?> clazz, String name, String scope) {
-      return registry.newCounter(clazz, name, scope);
+      return registry.counter(name(clazz, name, scope));
     }
   };
 
   // static metrics
 
-  protected final Counter metaCacheHits;
-  protected final Counter metaCacheMisses;
-  protected final CallTracker getTracker;
-  protected final CallTracker scanTracker;
-  protected final CallTracker appendTracker;
-  protected final CallTracker deleteTracker;
-  protected final CallTracker incrementTracker;
-  protected final CallTracker putTracker;
-  protected final CallTracker multiTracker;
-  protected final RunnerStats runnerStats;
-  protected final Counter metaCacheNumClearServer;
-  protected final Counter metaCacheNumClearRegion;
-  protected final Counter hedgedReadOps;
-  protected final Counter hedgedReadWin;
-  protected final Histogram concurrentCallsPerServerHist;
-  protected final Counter nsLookups;
-  protected final Counter nsLookupsFailed;
+  @VisibleForTesting protected final Counter metaCacheHits;
+  @VisibleForTesting protected final Counter metaCacheMisses;
+  @VisibleForTesting protected final CallTracker getTracker;
+  @VisibleForTesting protected final CallTracker scanTracker;
+  @VisibleForTesting protected final CallTracker appendTracker;
+  @VisibleForTesting protected final CallTracker deleteTracker;
+  @VisibleForTesting protected final CallTracker incrementTracker;
+  @VisibleForTesting protected final CallTracker putTracker;
+  @VisibleForTesting protected final CallTracker multiTracker;
+  @VisibleForTesting protected final RunnerStats runnerStats;
+  @VisibleForTesting protected final Counter metaCacheNumClearServer;
+  @VisibleForTesting protected final Counter metaCacheNumClearRegion;
+  @VisibleForTesting protected final Counter hedgedReadOps;
+  @VisibleForTesting protected final Counter hedgedReadWin;
+  @VisibleForTesting protected final Histogram concurrentCallsPerServerHist;
+  @VisibleForTesting protected final Histogram numActionsPerServerHist;
 
   // dynamic metrics
 
   // These maps are used to cache references to the metric instances that are managed by the
   // registry. I don't think their use perfectly removes redundant allocations, but it's
   // a big improvement over calling registry.newMetric each time.
-  protected final ConcurrentMap<String, Timer> rpcTimers =
+  @VisibleForTesting protected final ConcurrentMap<String, Timer> rpcTimers =
       new ConcurrentHashMap<>(CAPACITY, LOAD_FACTOR, CONCURRENCY_LEVEL);
-  protected final ConcurrentMap<String, Histogram> rpcHistograms =
+  @VisibleForTesting protected final ConcurrentMap<String, Histogram> rpcHistograms =
       new ConcurrentHashMap<>(CAPACITY * 2 /* tracking both request and response sizes */,
           LOAD_FACTOR, CONCURRENCY_LEVEL);
   private final ConcurrentMap<String, Counter> cacheDroppingExceptions =
     new ConcurrentHashMap<>(CAPACITY, LOAD_FACTOR, CONCURRENCY_LEVEL);
-  protected final ConcurrentMap<String, Counter>  rpcCounters =
-      new ConcurrentHashMap<>(CAPACITY, LOAD_FACTOR, CONCURRENCY_LEVEL);
 
-  public MetricsConnection(final ConnectionManager.HConnectionImplementation conn) {
-    this.scope = conn.toString();
-    this.registry = new MetricsRegistry();
-
-    this.registry.newGauge(getExecutorPoolName(),
+  MetricsConnection(String scope, Supplier<ThreadPoolExecutor> batchPool,
+      Supplier<ThreadPoolExecutor> metaPool) {
+    this.scope = scope;
+    this.registry = new MetricRegistry();
+    this.registry.register(getExecutorPoolName(),
         new RatioGauge() {
-          @Override protected double getNumerator() {
-            ThreadPoolExecutor batchPool = (ThreadPoolExecutor) conn.getCurrentBatchPool();
-            if (batchPool == null) {
-              return 0;
+          @Override
+          protected Ratio getRatio() {
+            ThreadPoolExecutor pool = batchPool.get();
+            if (pool == null) {
+              return Ratio.of(0, 0);
             }
-            return batchPool.getActiveCount();
-          }
-          @Override protected double getDenominator() {
-            ThreadPoolExecutor batchPool = (ThreadPoolExecutor) conn.getCurrentBatchPool();
-            if (batchPool == null) {
-              return 0;
-            }
-            return batchPool.getMaximumPoolSize();
+            return Ratio.of(pool.getActiveCount(), pool.getMaximumPoolSize());
           }
         });
-    this.registry.newGauge(getMetaPoolName(),
+    this.registry.register(getMetaPoolName(),
         new RatioGauge() {
-          @Override protected double getNumerator() {
-            ThreadPoolExecutor metaPool = (ThreadPoolExecutor) conn.getCurrentMetaLookupPool();
-            if (metaPool == null) {
-              return 0;
+          @Override
+          protected Ratio getRatio() {
+            ThreadPoolExecutor pool = metaPool.get();
+            if (pool == null) {
+              return Ratio.of(0, 0);
             }
-            return metaPool.getActiveCount();
-          }
-          @Override protected double getDenominator() {
-            ThreadPoolExecutor metaPool = (ThreadPoolExecutor) conn.getCurrentMetaLookupPool();
-            if (metaPool == null) {
-              return 0;
-            }
-            return metaPool.getMaximumPoolSize();
+            return Ratio.of(pool.getActiveCount(), pool.getMaximumPoolSize());
           }
         });
-    this.metaCacheHits = registry.newCounter(this.getClass(), "metaCacheHits", scope);
-    this.metaCacheMisses = registry.newCounter(this.getClass(), "metaCacheMisses", scope);
-    this.metaCacheNumClearServer = registry.newCounter(this.getClass(),
-      "metaCacheNumClearServer", scope);
-    this.metaCacheNumClearRegion = registry.newCounter(this.getClass(),
-      "metaCacheNumClearRegion", scope);
-    this.hedgedReadOps = registry.newCounter(this.getClass(), "hedgedReadOps", scope);
-    this.hedgedReadWin = registry.newCounter(this.getClass(), "hedgedReadWin", scope);
+    this.metaCacheHits = registry.counter(name(this.getClass(), "metaCacheHits", scope));
+    this.metaCacheMisses = registry.counter(name(this.getClass(), "metaCacheMisses", scope));
+    this.metaCacheNumClearServer = registry.counter(name(this.getClass(),
+      "metaCacheNumClearServer", scope));
+    this.metaCacheNumClearRegion = registry.counter(name(this.getClass(),
+      "metaCacheNumClearRegion", scope));
+    this.hedgedReadOps = registry.counter(name(this.getClass(), "hedgedReadOps", scope));
+    this.hedgedReadWin = registry.counter(name(this.getClass(), "hedgedReadWin", scope));
     this.getTracker = new CallTracker(this.registry, "Get", scope);
     this.scanTracker = new CallTracker(this.registry, "Scan", scope);
     this.appendTracker = new CallTracker(this.registry, "Mutate", "Append", scope);
@@ -361,30 +346,32 @@ public class MetricsConnection implements StatisticTrackable {
     this.putTracker = new CallTracker(this.registry, "Mutate", "Put", scope);
     this.multiTracker = new CallTracker(this.registry, "Multi", scope);
     this.runnerStats = new RunnerStats(this.registry);
-    this.concurrentCallsPerServerHist = registry.newHistogram(this.getClass(), 
-      "concurrentCallsPerServer", scope);
-    this.nsLookups = registry.newCounter(this.getClass(), NS_LOOKUPS, scope);
-    this.nsLookupsFailed = registry.newCounter(this.getClass(), NS_LOOKUPS_FAILED, scope);
+    this.concurrentCallsPerServerHist = registry.histogram(name(MetricsConnection.class,
+      "concurrentCallsPerServer", scope));
+    this.numActionsPerServerHist = registry.histogram(name(MetricsConnection.class,
+      "numActionsPerServer", scope));
 
-    this.reporter = new JmxReporter(this.registry);
+    this.reporter = JmxReporter.forRegistry(this.registry).build();
     this.reporter.start();
   }
 
-  final MetricName getExecutorPoolName() {
-    return new MetricName(getClass(), "executorPoolActiveThreads", scope);
+  @VisibleForTesting
+  final String getExecutorPoolName() {
+    return name(getClass(), "executorPoolActiveThreads", scope);
   }
 
-  final MetricName getMetaPoolName() {
-    return new MetricName(getClass(), "metaPoolActiveThreads", scope);
+  @VisibleForTesting
+  final String getMetaPoolName() {
+    return name(getClass(), "metaPoolActiveThreads", scope);
   }
 
-  MetricsRegistry getMetricsRegistry() {
+  @VisibleForTesting
+  MetricRegistry getMetricRegistry() {
     return registry;
   }
 
   public void shutdown() {
-    this.reporter.shutdown();
-    this.registry.shutdown();
+    this.reporter.stop();
   }
 
   /** Produce an instance of {@link CallStats} for clients to attach to RPCs. */
@@ -413,6 +400,11 @@ public class MetricsConnection implements StatisticTrackable {
     metaCacheNumClearRegion.inc();
   }
 
+  /** Increment the number of meta cache drops requested for individual region. */
+  public void incrMetaCacheNumClearRegion(int count) {
+    metaCacheNumClearRegion.inc(count);
+  }
+
   /** Increment the number of hedged read that have occurred. */
   public void incrHedgedReadOps() {
     hedgedReadOps.inc();
@@ -428,13 +420,9 @@ public class MetricsConnection implements StatisticTrackable {
     this.runnerStats.incrNormalRunners();
   }
 
-  /** Increment the number of delay runner counts. */
-  public void incrDelayRunners() {
+  /** Increment the number of delay runner counts and update delay interval of delay runner. */
+  public void incrDelayRunnersAndUpdateDelayInterval(long interval) {
     this.runnerStats.incrDelayRunners();
-  }
-
-  /** Update delay interval of delay runner. */
-  public void updateDelayInterval(long interval) {
     this.runnerStats.updateDelayInterval(interval);
   }
 
@@ -442,17 +430,12 @@ public class MetricsConnection implements StatisticTrackable {
    * Get a metric for {@code key} from {@code map}, or create it with {@code factory}.
    */
   private <T> T getMetric(String key, ConcurrentMap<String, T> map, NewMetric<T> factory) {
-    T t = map.get(key);
-    if (t == null) {
-      t = factory.newMetric(this.getClass(), key, scope);
-      T tmp = map.putIfAbsent(key, t);
-      t = (tmp == null) ? t : tmp;
-    }
-    return t;
+    return computeIfAbsent(map, key, () -> factory.newMetric(getClass(), key, scope));
   }
 
   /** Update call stats for non-critical-path methods */
-  private void updateRpcGeneric(String methodName, CallStats stats) {
+  private void updateRpcGeneric(MethodDescriptor method, CallStats stats) {
+    final String methodName = method.getService().getName() + "_" + method.getName();
     getMetric(DRTN_BASE + methodName, rpcTimers, timerFactory)
         .update(stats.getCallTimeMs(), TimeUnit.MILLISECONDS);
     getMetric(REQ_BASE + methodName, rpcHistograms, histogramFactory)
@@ -467,62 +450,68 @@ public class MetricsConnection implements StatisticTrackable {
     if (callsPerServer > 0) {
       concurrentCallsPerServerHist.update(callsPerServer);
     }
-    // Update the counter that tracks RPCs by type.
-    final String methodName = method.getService().getName() + "_" + method.getName();
-    getMetric(CNT_BASE + methodName, rpcCounters, counterFactory).inc();
     // this implementation is tied directly to protobuf implementation details. would be better
     // if we could dispatch based on something static, ie, request Message type.
     if (method.getService() == ClientService.getDescriptor()) {
       switch(method.getIndex()) {
-      case 0:
-        assert "Get".equals(method.getName());
-        getTracker.updateRpc(stats);
-        return;
-      case 1:
-        assert "Mutate".equals(method.getName());
-        final MutationType mutationType = ((MutateRequest) param).getMutation().getMutateType();
-        switch(mutationType) {
-        case APPEND:
-          appendTracker.updateRpc(stats);
+        case 0:
+          assert "Get".equals(method.getName());
+          getTracker.updateRpc(stats);
           return;
-        case DELETE:
-          deleteTracker.updateRpc(stats);
+        case 1:
+          assert "Mutate".equals(method.getName());
+          final MutationType mutationType = ((MutateRequest) param).getMutation().getMutateType();
+          switch(mutationType) {
+            case APPEND:
+              appendTracker.updateRpc(stats);
+              return;
+            case DELETE:
+              deleteTracker.updateRpc(stats);
+              return;
+            case INCREMENT:
+              incrementTracker.updateRpc(stats);
+              return;
+            case PUT:
+              putTracker.updateRpc(stats);
+              return;
+            default:
+              throw new RuntimeException("Unrecognized mutation type " + mutationType);
+          }
+        case 2:
+          assert "Scan".equals(method.getName());
+          scanTracker.updateRpc(stats);
           return;
-        case INCREMENT:
-          incrementTracker.updateRpc(stats);
-          return;
-        case PUT:
-          putTracker.updateRpc(stats);
+        case 3:
+          assert "BulkLoadHFile".equals(method.getName());
+          // use generic implementation
+          break;
+        case 4:
+          assert "PrepareBulkLoad".equals(method.getName());
+          // use generic implementation
+          break;
+        case 5:
+          assert "CleanupBulkLoad".equals(method.getName());
+          // use generic implementation
+          break;
+        case 6:
+          assert "ExecService".equals(method.getName());
+          // use generic implementation
+          break;
+        case 7:
+          assert "ExecRegionServerService".equals(method.getName());
+          // use generic implementation
+          break;
+        case 8:
+          assert "Multi".equals(method.getName());
+          numActionsPerServerHist.update(stats.getNumActionsPerServer());
+          multiTracker.updateRpc(stats);
           return;
         default:
-          throw new RuntimeException("Unrecognized mutation type " + mutationType);
-        }
-      case 2:
-        assert "Scan".equals(method.getName());
-        scanTracker.updateRpc(stats);
-        return;
-      case 3:
-        assert "BulkLoadHFile".equals(method.getName());
-        // use generic implementation
-        break;
-      case 4:
-        assert "ExecService".equals(method.getName());
-        // use generic implementation
-        break;
-      case 5:
-        assert "ExecRegionServerService".equals(method.getName());
-        // use generic implementation
-        break;
-      case 6:
-        assert "Multi".equals(method.getName());
-        multiTracker.updateRpc(stats);
-        return;
-      default:
-        throw new RuntimeException("Unrecognized ClientService RPC type " + method.getFullName());
+          throw new RuntimeException("Unrecognized ClientService RPC type " + method.getFullName());
       }
     }
     // Fallback to dynamic registry lookup for DDL methods.
-    updateRpcGeneric(methodName, stats);
+    updateRpcGeneric(method, stats);
   }
 
   public void incrCacheDroppingExceptions(Object exception) {
@@ -530,12 +519,4 @@ public class MetricsConnection implements StatisticTrackable {
       (exception == null? UNKNOWN_EXCEPTION : exception.getClass().getSimpleName()),
       cacheDroppingExceptions, counterFactory).inc();
   }
-
-  public void incrNsLookups() {
-    this.nsLookups.inc();
-  }
-
-  public void incrNsLookupsFailed() {
-    this.nsLookupsFailed.inc();
-  }
 }
