@@ -25,76 +25,93 @@ import java.net.SocketTimeoutException;
 import java.util.Comparator;
 import java.util.HashMap;
 import java.util.Map;
+import java.util.Objects;
 import java.util.Random;
 import java.util.SortedMap;
+import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ConcurrentSkipListMap;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicLong;
-
-import org.apache.hadoop.hbase.util.ByteStringer;
-import org.apache.commons.lang.NotImplementedException;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.apache.commons.lang3.NotImplementedException;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.conf.Configured;
+import org.apache.hadoop.hbase.CellComparatorImpl;
 import org.apache.hadoop.hbase.DoNotRetryIOException;
+import org.apache.hadoop.hbase.HBaseClassTestRule;
 import org.apache.hadoop.hbase.HBaseConfiguration;
 import org.apache.hadoop.hbase.HConstants;
 import org.apache.hadoop.hbase.HRegionInfo;
 import org.apache.hadoop.hbase.HRegionLocation;
-import org.apache.hadoop.hbase.RegionLocations;
 import org.apache.hadoop.hbase.KeyValue;
+import org.apache.hadoop.hbase.MetaCellComparator;
+import org.apache.hadoop.hbase.MetaTableAccessor;
+import org.apache.hadoop.hbase.RegionLocations;
 import org.apache.hadoop.hbase.RegionTooBusyException;
 import org.apache.hadoop.hbase.ServerName;
-import org.apache.hadoop.hbase.testclassification.SmallTests;
 import org.apache.hadoop.hbase.TableName;
-import org.apache.hadoop.hbase.protobuf.generated.CellProtos;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos.BulkLoadHFileRequest;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos.BulkLoadHFileResponse;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos.ClientService;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos.ClientService.BlockingInterface;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos.CoprocessorServiceRequest;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos.CoprocessorServiceResponse;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos.GetRequest;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos.GetResponse;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos.MultiRequest;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos.MultiResponse;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos.MutateRequest;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos.MutateResponse;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos.RegionAction;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos.RegionActionResult;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos.ResultOrException;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos.ScanRequest;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos.ScanResponse;
-import org.apache.hadoop.hbase.protobuf.generated.HBaseProtos.RegionSpecifier.RegionSpecifierType;
 import org.apache.hadoop.hbase.regionserver.RegionServerStoppedException;
 import org.apache.hadoop.hbase.security.User;
+import org.apache.hadoop.hbase.testclassification.ClientTests;
+import org.apache.hadoop.hbase.testclassification.SmallTests;
 import org.apache.hadoop.hbase.util.Bytes;
 import org.apache.hadoop.hbase.util.Pair;
 import org.apache.hadoop.hbase.util.Threads;
 import org.apache.hadoop.util.Tool;
 import org.apache.hadoop.util.ToolRunner;
 import org.junit.Before;
+import org.junit.ClassRule;
 import org.junit.Ignore;
 import org.junit.Test;
 import org.junit.experimental.categories.Category;
 import org.mockito.Mockito;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
-import com.google.common.base.Stopwatch;
-import com.google.protobuf.ByteString;
-import com.google.protobuf.RpcController;
-import com.google.protobuf.ServiceException;
+import org.apache.hbase.thirdparty.com.google.common.base.Stopwatch;
+import org.apache.hbase.thirdparty.com.google.protobuf.ByteString;
+import org.apache.hbase.thirdparty.com.google.protobuf.RpcController;
+import org.apache.hbase.thirdparty.com.google.protobuf.ServiceException;
+import org.apache.hbase.thirdparty.com.google.protobuf.UnsafeByteOperations;
+
+import org.apache.hadoop.hbase.shaded.protobuf.generated.CellProtos;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.BulkLoadHFileRequest;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.BulkLoadHFileResponse;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.CleanupBulkLoadRequest;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.CleanupBulkLoadResponse;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.ClientService;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.ClientService.BlockingInterface;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.CoprocessorServiceRequest;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.CoprocessorServiceResponse;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.GetRequest;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.GetResponse;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.MultiRequest;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.MultiResponse;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.MutateRequest;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.MutateResponse;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.PrepareBulkLoadRequest;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.PrepareBulkLoadResponse;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.RegionAction;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.RegionActionResult;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.ResultOrException;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.ScanRequest;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.ClientProtos.ScanResponse;
+import org.apache.hadoop.hbase.shaded.protobuf.generated.HBaseProtos.RegionSpecifier.RegionSpecifierType;
 
 /**
  * Test client behavior w/o setting up a cluster.
  * Mock up cluster emissions.
  */
-@Category(SmallTests.class)
+@Category({ClientTests.class, SmallTests.class})
 public class TestClientNoCluster extends Configured implements Tool {
-  private static final Log LOG = LogFactory.getLog(TestClientNoCluster.class);
+
+  @ClassRule
+  public static final HBaseClassTestRule CLASS_RULE =
+      HBaseClassTestRule.forClass(TestClientNoCluster.class);
+
+  private static final Logger LOG = LoggerFactory.getLogger(TestClientNoCluster.class);
   private Configuration conf;
   public static final ServerName META_SERVERNAME =
       ServerName.valueOf("meta.example.org", 16010, 12345);
@@ -102,51 +119,31 @@ public class TestClientNoCluster extends Configured implements Tool {
   @Before
   public void setUp() throws Exception {
     this.conf = HBaseConfiguration.create();
-    // Run my HConnection overrides.  Use my little HConnectionImplementation below which
+    // Run my Connection overrides.  Use my little ConnectionImplementation below which
     // allows me insert mocks and also use my Registry below rather than the default zk based
     // one so tests run faster and don't have zk dependency.
-    this.conf.set("hbase.client.registry.impl", SimpleConnectionRegistry.class.getName());
+    this.conf.set("hbase.client.registry.impl", SimpleRegistry.class.getName());
   }
 
   /**
    * Simple cluster registry inserted in place of our usual zookeeper based one.
    */
-  static class SimpleConnectionRegistry implements ConnectionRegistry {
+  static class SimpleRegistry extends DoNothingConnectionRegistry {
     final ServerName META_HOST = META_SERVERNAME;
 
-    @Override
-    public void init(Connection connection) {
+    public SimpleRegistry(Configuration conf) {
+      super(conf);
     }
 
     @Override
-    public ServerName getActiveMaster() {
-      return null;
+    public CompletableFuture<RegionLocations> getMetaRegionLocations() {
+      return CompletableFuture.completedFuture(new RegionLocations(
+          new HRegionLocation(RegionInfoBuilder.FIRST_META_REGIONINFO, META_HOST)));
     }
 
     @Override
-    public RegionLocations getMetaRegionLocations() throws IOException {
-      return new RegionLocations(
-        new HRegionLocation(HRegionInfo.FIRST_META_REGIONINFO, META_HOST));
-    }
-
-    @Override
-    public String getClusterId() {
-      return HConstants.CLUSTER_ID_DEFAULT;
-    }
-
-    @Override
-    public boolean isTableOnlineState(TableName tableName, boolean enabled)
-    throws IOException {
-      return enabled;
-    }
-
-    @Override
-    public int getCurrentNrHRS() throws IOException {
-      return 1;
-    }
-
-    @Override
-    public void close() {
+    public CompletableFuture<String> getClusterId() {
+      return CompletableFuture.completedFuture(HConstants.CLUSTER_ID_DEFAULT);
     }
   }
 
@@ -160,7 +157,8 @@ public class TestClientNoCluster extends Configured implements Tool {
     Configuration localConfig = HBaseConfiguration.create(this.conf);
     // This override mocks up our exists/get call to throw a RegionServerStoppedException.
     localConfig.set("hbase.client.connection.impl", RpcTimeoutConnection.class.getName());
-    Table table = new HTable(localConfig, TableName.META_TABLE_NAME);
+    Connection connection = ConnectionFactory.createConnection(localConfig);
+    Table table = connection.getTable(TableName.META_TABLE_NAME);
     Throwable t = null;
     LOG.info("Start");
     try {
@@ -176,6 +174,7 @@ public class TestClientNoCluster extends Configured implements Tool {
     } finally {
       table.close();
     }
+    connection.close();
     LOG.info("Stop");
     assertTrue(t != null);
   }
@@ -197,7 +196,8 @@ public class TestClientNoCluster extends Configured implements Tool {
     // and it has expired.  Otherwise, if this functionality is broke, all retries will be run --
     // all ten of them -- and we'll get the RetriesExhaustedException exception.
     localConfig.setInt(HConstants.HBASE_CLIENT_META_OPERATION_TIMEOUT, pause - 1);
-    Table table = new HTable(localConfig, TableName.META_TABLE_NAME);
+    Connection connection = ConnectionFactory.createConnection(localConfig);
+    Table table = connection.getTable(TableName.META_TABLE_NAME);
     Throwable t = null;
     try {
       // An exists call turns into a get w/ a flag.
@@ -211,16 +211,17 @@ public class TestClientNoCluster extends Configured implements Tool {
       fail();
     } finally {
       table.close();
+      connection.close();
     }
     assertTrue(t != null);
   }
 
   @Test
-  public void testDoNotRetryMetaScanner() throws IOException {
+  public void testDoNotRetryMetaTableAccessor() throws IOException {
     this.conf.set("hbase.client.connection.impl",
       RegionServerStoppedOnScannerOpenConnection.class.getName());
     try (Connection connection = ConnectionFactory.createConnection(conf)) {
-      MetaScanner.metaScan(connection, null);
+      MetaTableAccessor.fullScanRegions(connection);
     }
   }
 
@@ -231,16 +232,18 @@ public class TestClientNoCluster extends Configured implements Tool {
     // Go against meta else we will try to find first region for the table on construction which
     // means we'll have to do a bunch more mocking.  Tests that go against meta only should be
     // good for a bit of testing.
-    Table table = new HTable(this.conf, TableName.META_TABLE_NAME);
+    Connection connection = ConnectionFactory.createConnection(this.conf);
+    Table table = connection.getTable(TableName.META_TABLE_NAME);
     ResultScanner scanner = table.getScanner(HConstants.CATALOG_FAMILY);
     try {
       Result result = null;
       while ((result = scanner.next()) != null) {
-        LOG.info(result);
+        LOG.info(Objects.toString(result));
       }
     } finally {
       scanner.close();
       table.close();
+      connection.close();
     }
   }
 
@@ -251,48 +254,18 @@ public class TestClientNoCluster extends Configured implements Tool {
     // Go against meta else we will try to find first region for the table on construction which
     // means we'll have to do a bunch more mocking.  Tests that go against meta only should be
     // good for a bit of testing.
-    Table table = new HTable(this.conf, TableName.META_TABLE_NAME);
+    Connection connection = ConnectionFactory.createConnection(conf);
+    Table table = connection.getTable(TableName.META_TABLE_NAME);
     ResultScanner scanner = table.getScanner(HConstants.CATALOG_FAMILY);
     try {
       Result result = null;
       while ((result = scanner.next()) != null) {
-        LOG.info(result);
+        LOG.info(Objects.toString(result));
       }
     } finally {
       scanner.close();
       table.close();
-    }
-  }
-
-  /**
-   * Override to shutdown going to zookeeper for cluster id and meta location.
-   */
-  static class ScanOpenNextThenExceptionThenRecoverConnection
-  extends ConnectionManager.HConnectionImplementation {
-    final ClientService.BlockingInterface stub;
-
-    ScanOpenNextThenExceptionThenRecoverConnection(Configuration conf,
-        boolean managed, ExecutorService pool) throws IOException {
-      super(conf, managed);
-      // Mock up my stub so open scanner returns a scanner id and then on next, we throw
-      // exceptions for three times and then after that, we return no more to scan.
-      this.stub = Mockito.mock(ClientService.BlockingInterface.class);
-      long sid = 12345L;
-      try {
-        Mockito.when(stub.scan((RpcController)Mockito.any(),
-            (ClientProtos.ScanRequest)Mockito.any())).
-          thenReturn(ClientProtos.ScanResponse.newBuilder().setScannerId(sid).build()).
-          thenThrow(new ServiceException(new RegionServerStoppedException("From Mockito"))).
-          thenReturn(ClientProtos.ScanResponse.newBuilder().setScannerId(sid).
-              setMoreResults(false).build());
-      } catch (ServiceException e) {
-        throw new IOException(e);
-      }
-    }
-
-    @Override
-    public BlockingInterface getClient(ServerName sn) throws IOException {
-      return this.stub;
+      connection.close();
     }
   }
 
@@ -326,12 +299,12 @@ public class TestClientNoCluster extends Configured implements Tool {
    * Override to shutdown going to zookeeper for cluster id and meta location.
    */
   static class RegionServerStoppedOnScannerOpenConnection
-  extends ConnectionManager.HConnectionImplementation {
+  extends ConnectionImplementation {
     final ClientService.BlockingInterface stub;
 
-    RegionServerStoppedOnScannerOpenConnection(Configuration conf, boolean managed,
+    RegionServerStoppedOnScannerOpenConnection(Configuration conf,
         ExecutorService pool, User user) throws IOException {
-      super(conf, managed);
+      super(conf, pool, user);
       // Mock up my stub so open scanner returns a scanner id and then on next, we throw
       // exceptions for three times and then after that, we return no more to scan.
       this.stub = Mockito.mock(ClientService.BlockingInterface.class);
@@ -358,12 +331,12 @@ public class TestClientNoCluster extends Configured implements Tool {
    * Override to check we are setting rpc timeout right.
    */
   static class RpcTimeoutConnection
-  extends ConnectionManager.HConnectionImplementation {
+  extends ConnectionImplementation {
     final ClientService.BlockingInterface stub;
 
-    RpcTimeoutConnection(Configuration conf, boolean managed, ExecutorService pool, User user)
+    RpcTimeoutConnection(Configuration conf, ExecutorService pool, User user)
     throws IOException {
-      super(conf, managed);
+      super(conf, pool, user);
       // Mock up my stub so an exists call -- which turns into a get -- throws an exception
       this.stub = Mockito.mock(ClientService.BlockingInterface.class);
       try {
@@ -385,7 +358,7 @@ public class TestClientNoCluster extends Configured implements Tool {
    * Fake many regionservers and many regions on a connection implementation.
    */
   static class ManyServersManyRegionsConnection
-  extends ConnectionManager.HConnectionImplementation {
+  extends ConnectionImplementation {
     // All access should be synchronized
     final Map<ServerName, ClientService.BlockingInterface> serversByClient;
 
@@ -396,13 +369,12 @@ public class TestClientNoCluster extends Configured implements Tool {
     final AtomicLong sequenceids = new AtomicLong(0);
     private final Configuration conf;
 
-    ManyServersManyRegionsConnection(Configuration conf, boolean managed,
+    ManyServersManyRegionsConnection(Configuration conf,
         ExecutorService pool, User user)
     throws IOException {
-      super(conf, managed, pool, user);
+      super(conf, pool, user);
       int serverCount = conf.getInt("hbase.test.servers", 10);
-      this.serversByClient =
-        new HashMap<ServerName, ClientService.BlockingInterface>(serverCount);
+      this.serversByClient = new HashMap<>(serverCount);
       this.meta = makeMeta(Bytes.toBytes(
         conf.get("hbase.test.tablename", Bytes.toString(BIG_USER_TABLE))),
         conf.getInt("hbase.test.regions", 100),
@@ -495,7 +467,7 @@ public class TestClientNoCluster extends Configured implements Tool {
     @Override
     public MutateResponse mutate(RpcController controller,
         MutateRequest request) throws ServiceException {
-      throw new NotImplementedException();
+      throw new NotImplementedException(HConstants.NOT_IMPLEMENTED);
     }
 
     @Override
@@ -510,14 +482,14 @@ public class TestClientNoCluster extends Configured implements Tool {
     public BulkLoadHFileResponse bulkLoadHFile(
         RpcController controller, BulkLoadHFileRequest request)
         throws ServiceException {
-      throw new NotImplementedException();
+      throw new NotImplementedException(HConstants.NOT_IMPLEMENTED);
     }
 
     @Override
     public CoprocessorServiceResponse execService(
         RpcController controller, CoprocessorServiceRequest request)
         throws ServiceException {
-      throw new NotImplementedException();
+      throw new NotImplementedException(HConstants.NOT_IMPLEMENTED);
     }
 
     @Override
@@ -539,7 +511,19 @@ public class TestClientNoCluster extends Configured implements Tool {
     @Override
     public CoprocessorServiceResponse execRegionServerService(RpcController controller,
         CoprocessorServiceRequest request) throws ServiceException {
-      throw new NotImplementedException();
+      throw new NotImplementedException(HConstants.NOT_IMPLEMENTED);
+    }
+
+    @Override
+    public PrepareBulkLoadResponse prepareBulkLoad(RpcController controller,
+        PrepareBulkLoadRequest request) throws ServiceException {
+      throw new NotImplementedException(HConstants.NOT_IMPLEMENTED);
+    }
+
+    @Override
+    public CleanupBulkLoadResponse cleanupBulkLoad(RpcController controller,
+        CleanupBulkLoadRequest request) throws ServiceException {
+      throw new NotImplementedException(HConstants.NOT_IMPLEMENTED);
     }
   }
 
@@ -556,7 +540,7 @@ public class TestClientNoCluster extends Configured implements Tool {
       if (max <= 0) break;
       if (++count > max) break;
       HRegionInfo hri = e.getValue().getFirst();
-      ByteString row = ByteStringer.wrap(hri.getRegionName());
+      ByteString row = UnsafeByteOperations.unsafeWrap(hri.getRegionName());
       resultBuilder.clear();
       resultBuilder.addCell(getRegionInfo(row, hri));
       resultBuilder.addCell(getServer(row, e.getValue().getSecond()));
@@ -577,14 +561,6 @@ public class TestClientNoCluster extends Configured implements Tool {
     ClientProtos.Result.Builder resultBuilder = ClientProtos.Result.newBuilder();
     ByteString row = request.getGet().getRow();
     Pair<HRegionInfo, ServerName> p = meta.get(row.toByteArray());
-    if (p == null) {
-      if (request.getGet().getClosestRowBefore()) {
-        byte [] bytes = row.toByteArray();
-        SortedMap<byte [], Pair<HRegionInfo, ServerName>> head =
-          bytes != null? meta.headMap(bytes): meta;
-        p = head == null? null: head.get(head.lastKey());
-      }
-    }
     if (p != null) {
       resultBuilder.addCell(getRegionInfo(row, p.getFirst()));
       resultBuilder.addCell(getServer(row, p.getSecond()));
@@ -611,11 +587,11 @@ public class TestClientNoCluster extends Configured implements Tool {
   }
 
   private final static ByteString CATALOG_FAMILY_BYTESTRING =
-      ByteStringer.wrap(HConstants.CATALOG_FAMILY);
+      UnsafeByteOperations.unsafeWrap(HConstants.CATALOG_FAMILY);
   private final static ByteString REGIONINFO_QUALIFIER_BYTESTRING =
-      ByteStringer.wrap(HConstants.REGIONINFO_QUALIFIER);
+      UnsafeByteOperations.unsafeWrap(HConstants.REGIONINFO_QUALIFIER);
   private final static ByteString SERVER_QUALIFIER_BYTESTRING =
-      ByteStringer.wrap(HConstants.SERVER_QUALIFIER);
+      UnsafeByteOperations.unsafeWrap(HConstants.SERVER_QUALIFIER);
 
   static CellProtos.Cell.Builder getBaseCellBuilder(final ByteString row) {
     CellProtos.Cell.Builder cellBuilder = CellProtos.Cell.newBuilder();
@@ -628,22 +604,23 @@ public class TestClientNoCluster extends Configured implements Tool {
   static CellProtos.Cell getRegionInfo(final ByteString row, final HRegionInfo hri) {
     CellProtos.Cell.Builder cellBuilder = getBaseCellBuilder(row);
     cellBuilder.setQualifier(REGIONINFO_QUALIFIER_BYTESTRING);
-    cellBuilder.setValue(ByteStringer.wrap(hri.toByteArray()));
+    cellBuilder.setValue(UnsafeByteOperations.unsafeWrap(hri.toByteArray()));
     return cellBuilder.build();
   }
 
   static CellProtos.Cell getServer(final ByteString row, final ServerName sn) {
     CellProtos.Cell.Builder cellBuilder = getBaseCellBuilder(row);
     cellBuilder.setQualifier(SERVER_QUALIFIER_BYTESTRING);
-    cellBuilder.setValue(ByteString.copyFromUtf8(sn.getHostAndPort()));
+    cellBuilder.setValue(ByteString.copyFromUtf8(sn.getAddress().toString()));
     return cellBuilder.build();
   }
 
   static CellProtos.Cell getStartCode(final ByteString row) {
     CellProtos.Cell.Builder cellBuilder = getBaseCellBuilder(row);
-    cellBuilder.setQualifier(ByteStringer.wrap(HConstants.STARTCODE_QUALIFIER));
+    cellBuilder.setQualifier(UnsafeByteOperations.unsafeWrap(HConstants.STARTCODE_QUALIFIER));
     // TODO:
-    cellBuilder.setValue(ByteStringer.wrap(Bytes.toBytes(META_SERVERNAME.getStartcode())));
+    cellBuilder.setValue(UnsafeByteOperations.unsafeWrap(
+        Bytes.toBytes(META_SERVERNAME.getStartcode())));
     return cellBuilder.build();
   }
 
@@ -706,10 +683,10 @@ public class TestClientNoCluster extends Configured implements Tool {
    * Comparator for meta row keys.
    */
   private static class MetaRowsComparator implements Comparator<byte []> {
-    private final KeyValue.KVComparator delegate = new KeyValue.MetaComparator();
+    private final CellComparatorImpl delegate = MetaCellComparator.META_COMPARATOR;
     @Override
     public int compare(byte[] left, byte[] right) {
-      return delegate.compareRows(left, 0, left.length, right, 0, right.length);
+      return delegate.compareRows(new KeyValue.KeyOnlyKeyValue(left), right, 0, right.length);
     }
   }
 
@@ -722,14 +699,13 @@ public class TestClientNoCluster extends Configured implements Tool {
       final int regionCount, final long namespaceSpan, final int serverCount) {
     // I need a comparator for meta rows so we sort properly.
     SortedMap<byte [], Pair<HRegionInfo, ServerName>> meta =
-      new ConcurrentSkipListMap<byte[], Pair<HRegionInfo,ServerName>>(new MetaRowsComparator());
+      new ConcurrentSkipListMap<>(new MetaRowsComparator());
     HRegionInfo [] hris = makeHRegionInfos(tableName, regionCount, namespaceSpan);
     ServerName [] serverNames = makeServerNames(serverCount);
     int per = regionCount / serverCount;
     int count = 0;
     for (HRegionInfo hri: hris) {
-      Pair<HRegionInfo, ServerName> p =
-        new Pair<HRegionInfo, ServerName>(hri, serverNames[count++ / per]);
+      Pair<HRegionInfo, ServerName> p = new Pair<>(hri, serverNames[count++ / per]);
       meta.put(hri.getRegionName(), p);
     }
     return meta;
@@ -752,14 +728,13 @@ public class TestClientNoCluster extends Configured implements Tool {
     TableName tableName = TableName.valueOf(BIG_USER_TABLE);
     if (get) {
       try (Table table = sharedConnection.getTable(tableName)){
-        Stopwatch stopWatch = new Stopwatch();
-        stopWatch.start();
+        Stopwatch stopWatch = Stopwatch.createStarted();
         for (int i = 0; i < namespaceSpan; i++) {
           byte [] b = format(rd.nextLong());
           Get g = new Get(b);
           table.get(g);
           if (i % printInterval == 0) {
-            LOG.info("Get " + printInterval + "/" + stopWatch.elapsedMillis());
+            LOG.info("Get " + printInterval + "/" + stopWatch.elapsed(java.util.concurrent.TimeUnit.MILLISECONDS));
             stopWatch.reset();
             stopWatch.start();
           }
@@ -769,15 +744,14 @@ public class TestClientNoCluster extends Configured implements Tool {
       }
     } else {
       try (BufferedMutator mutator = sharedConnection.getBufferedMutator(tableName)) {
-        Stopwatch stopWatch = new Stopwatch();
-        stopWatch.start();
+        Stopwatch stopWatch = Stopwatch.createStarted();
         for (int i = 0; i < namespaceSpan; i++) {
           byte [] b = format(rd.nextLong());
           Put p = new Put(b);
-          p.add(HConstants.CATALOG_FAMILY, b, b);
+          p.addColumn(HConstants.CATALOG_FAMILY, b, b);
           mutator.mutate(p);
           if (i % printInterval == 0) {
-            LOG.info("Put " + printInterval + "/" + stopWatch.elapsedMillis());
+            LOG.info("Put " + printInterval + "/" + stopWatch.elapsed(java.util.concurrent.TimeUnit.MILLISECONDS));
             stopWatch.reset();
             stopWatch.start();
           }
@@ -811,7 +785,7 @@ public class TestClientNoCluster extends Configured implements Tool {
     getConf().set("hbase.client.connection.impl",
       ManyServersManyRegionsConnection.class.getName());
     // Use simple kv registry rather than zk
-    getConf().set("hbase.client.registry.impl", SimpleConnectionRegistry.class.getName());
+    getConf().set("hbase.client.registry.impl", SimpleRegistry.class.getName());
     // When to report fails.  Default is we report the 10th.  This means we'll see log everytime
     // an exception is thrown -- usually RegionTooBusyException when we have more than
     // hbase.test.multi.too.many requests outstanding at any time.
@@ -829,7 +803,8 @@ public class TestClientNoCluster extends Configured implements Tool {
 
     // Have them all share the same connection so they all share the same instance of
     // ManyServersManyRegionsConnection so I can keep an eye on how many requests by server.
-    final ExecutorService pool = Executors.newCachedThreadPool(Threads.getNamedThreadFactory("p"));
+    final ExecutorService pool = Executors.newCachedThreadPool(
+        Threads.newDaemonThreadFactory("p"));
       // Executors.newFixedThreadPool(servers * 10, Threads.getNamedThreadFactory("p"));
     // Share a connection so I can keep counts in the 'server' on concurrency.
     final Connection sharedConnection = ConnectionFactory.createConnection(getConf()/*, pool*/);
