@@ -19,18 +19,25 @@ package org.apache.hadoop.hbase.util;
 import java.io.ByteArrayOutputStream;
 import java.io.DataInput;
 import java.io.DataInputStream;
+import java.io.DataOutput;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
+import java.math.BigDecimal;
+import java.math.BigInteger;
 import java.nio.ByteBuffer;
+import java.util.Arrays;
 
-import org.apache.hadoop.hbase.classification.InterfaceAudience;
-import org.apache.hadoop.hbase.classification.InterfaceStability;
+import org.apache.hadoop.hbase.io.ByteBufferWriter;
+import org.apache.hadoop.hbase.io.util.StreamUtils;
+import org.apache.hadoop.hbase.nio.ByteBuff;
 import org.apache.hadoop.io.IOUtils;
 import org.apache.hadoop.io.WritableUtils;
-
+import org.apache.yetus.audience.InterfaceAudience;
 import sun.nio.ch.DirectBuffer;
 
+import org.apache.hbase.thirdparty.com.google.common.annotations.VisibleForTesting;
+
 /**
  * Utility functions for working with byte buffers, such as reading/writing
  * variable-length long numbers.
@@ -39,19 +46,19 @@ import sun.nio.ch.DirectBuffer;
 @SuppressWarnings("restriction")
 @Deprecated
 @InterfaceAudience.Public
-@InterfaceStability.Evolving
 public final class ByteBufferUtils {
   // "Compressed integer" serialization helper constants.
   public final static int VALUE_MASK = 0x7f;
   public final static int NEXT_BIT_SHIFT = 7;
   public final static int NEXT_BIT_MASK = 1 << 7;
-  @InterfaceAudience.Private
-  static final boolean UNSAFE_AVAIL = UnsafeAvailChecker.isAvailable();
-  public static final boolean UNSAFE_UNALIGNED = UnsafeAvailChecker.unaligned();
+  @VisibleForTesting
+  final static boolean UNSAFE_AVAIL = UnsafeAvailChecker.isAvailable();
+  public final static boolean UNSAFE_UNALIGNED = UnsafeAvailChecker.unaligned();
 
   private ByteBufferUtils() {
   }
 
+
   static abstract class Comparer {
     abstract int compareTo(byte [] buf1, int o1, int l1, ByteBuffer buf2, int o2, int l2);
     abstract int compareTo(ByteBuffer buf1, int o1, int l1, ByteBuffer buf2, int o2, int l2);
@@ -344,25 +351,39 @@ public final class ByteBufferUtils {
     }
   }
 
-  /**
-   * Similar to {@link WritableUtils#readVLong(DataInput)} but reads from a
-   * {@link ByteBuffer}.
-   */
-  public static long readVLong(ByteBuffer in) {
-    byte firstByte = in.get();
+  private interface ByteVisitor {
+    byte get();
+  }
+
+  private static long readVLong(ByteVisitor visitor) {
+    byte firstByte = visitor.get();
     int len = WritableUtils.decodeVIntSize(firstByte);
     if (len == 1) {
       return firstByte;
     }
     long i = 0;
-    for (int idx = 0; idx < len-1; idx++) {
-      byte b = in.get();
+    for (int idx = 0; idx < len - 1; idx++) {
+      byte b = visitor.get();
       i = i << 8;
       i = i | (b & 0xFF);
     }
     return (WritableUtils.isNegativeVInt(firstByte) ? (i ^ -1L) : i);
   }
 
+  /**
+   * Similar to {@link WritableUtils#readVLong(DataInput)} but reads from a {@link ByteBuffer}.
+   */
+  public static long readVLong(ByteBuffer in) {
+    return readVLong(in::get);
+  }
+
+  /**
+   * Similar to {@link WritableUtils#readVLong(java.io.DataInput)} but reads from a
+   * {@link ByteBuff}.
+   */
+  public static long readVLong(ByteBuff in) {
+    return readVLong(in::get);
+  }
 
   /**
    * Put in buffer integer using 7 bit encoding. For each written byte:
@@ -397,18 +418,23 @@ public final class ByteBufferUtils {
     */
    public static void putInt(OutputStream out, final int value)
        throws IOException {
-     for (int i = Bytes.SIZEOF_INT - 1; i >= 0; --i) {
-       out.write((byte) (value >>> (i * 8)));
+     // We have writeInt in ByteBufferOutputStream so that it can directly write
+     // int to underlying
+     // ByteBuffer in one step.
+     if (out instanceof ByteBufferWriter) {
+       ((ByteBufferWriter) out).writeInt(value);
+     } else {
+       StreamUtils.writeInt(out, value);
      }
    }
 
-   public static byte toByte(ByteBuffer buffer, int offset) {
-     if (UNSAFE_AVAIL) {
-       return UnsafeAccess.toByte(buffer, offset);
-     } else {
-       return buffer.get(offset);
-     }
-   }
+  public static byte toByte(ByteBuffer buffer, int offset) {
+    if (UNSAFE_AVAIL) {
+      return UnsafeAccess.toByte(buffer, offset);
+    } else {
+      return buffer.get(offset);
+    }
+  }
 
   /**
    * Copy the data to the output stream and update position in buffer.
@@ -433,7 +459,31 @@ public final class ByteBufferUtils {
    */
   public static void copyBufferToStream(OutputStream out, ByteBuffer in,
       int offset, int length) throws IOException {
-    if (in.hasArray()) {
+    if (out instanceof ByteBufferWriter) {
+      ((ByteBufferWriter) out).write(in, offset, length);
+    } else if (in.hasArray()) {
+      out.write(in.array(), in.arrayOffset() + offset, length);
+    } else {
+      for (int i = 0; i < length; ++i) {
+        out.write(toByte(in, offset + i));
+      }
+    }
+  }
+
+  /**
+   * Copy data from a buffer to an output stream. Does not update the position
+   * in the buffer.
+   * @param out the output stream to write bytes to
+   * @param in the buffer to read bytes from
+   * @param offset the offset in the buffer (from the buffer's array offset)
+   *      to start copying bytes from
+   * @param length the number of bytes to copy
+   */
+  public static void copyBufferToStream(DataOutput out, ByteBuffer in, int offset, int length)
+      throws IOException {
+    if (out instanceof ByteBufferWriter) {
+      ((ByteBufferWriter) out).write(in, offset, length);
+    } else if (in.hasArray()) {
       out.write(in.array(), in.arrayOffset() + offset, length);
     } else {
       for (int i = 0; i < length; ++i) {
@@ -452,6 +502,15 @@ public final class ByteBufferUtils {
     return fitInBytes;
   }
 
+  public static int putByte(ByteBuffer buffer, int offset, byte b) {
+    if (UNSAFE_AVAIL) {
+      return UnsafeAccess.putByte(buffer, offset, b);
+    } else {
+      buffer.put(offset, b);
+      return offset + 1;
+    }
+  }
+
   /**
    * Check how many bytes are required to store value.
    * @param value Value which size will be tested.
@@ -604,26 +663,25 @@ public final class ByteBufferUtils {
   }
 
   /**
-   * Copy from one buffer to another from given offset.
-   * <p>
-   * Note : This will advance the position marker of {@code out} but not change the position maker
-   * for {@code in}
-   * @param out destination buffer
+   * Copy one buffer's whole data to another. Write starts at the current position of 'out' buffer.
+   * Note : This will advance the position marker of {@code out} and also change the position maker
+   * for {@code in}.
    * @param in source buffer
-   * @param sourceOffset offset in the source buffer
-   * @param length how many bytes to copy
+   * @param out destination buffer
    */
-  public static void copyFromBufferToBuffer(ByteBuffer out,
-      ByteBuffer in, int sourceOffset, int length) {
+  public static void copyFromBufferToBuffer(ByteBuffer in, ByteBuffer out) {
     if (in.hasArray() && out.hasArray()) {
-      System.arraycopy(in.array(), sourceOffset + in.arrayOffset(),
-          out.array(), out.position() +
-          out.arrayOffset(), length);
-      skip(out, length);
+      int length = in.remaining();
+      System.arraycopy(in.array(), in.arrayOffset(), out.array(), out.arrayOffset(), length);
+      out.position(out.position() + length);
+      in.position(in.limit());
+    } else if (UNSAFE_AVAIL) {
+      int length = in.remaining();
+      UnsafeAccess.copy(in, in.position(), out, out.position(), length);
+      out.position(out.position() + length);
+      in.position(in.limit());
     } else {
-      for (int i = 0; i < length; ++i) {
-        out.put(in.get(sourceOffset + i));
-      }
+      out.put(in);
     }
   }
 
@@ -636,7 +694,7 @@ public final class ByteBufferUtils {
    * @param destinationOffset
    * @param length
    */
-  public static void copyFromBufferToBuffer(ByteBuffer out, ByteBuffer in, int sourceOffset,
+  public static void copyFromBufferToBuffer(ByteBuffer in, ByteBuffer out, int sourceOffset,
       int destinationOffset, int length) {
     if (in.hasArray() && out.hasArray()) {
       System.arraycopy(in.array(), sourceOffset + in.arrayOffset(), out.array(), out.arrayOffset()
@@ -644,9 +702,38 @@ public final class ByteBufferUtils {
     } else if (UNSAFE_AVAIL) {
       UnsafeAccess.copy(in, sourceOffset, out, destinationOffset, length);
     } else {
-      for (int i = 0; i < length; ++i) {
-        out.put((destinationOffset + i), in.get(sourceOffset + i));
-      }
+      ByteBuffer outDup = out.duplicate();
+      outDup.position(destinationOffset);
+      ByteBuffer inDup = in.duplicate();
+      inDup.position(sourceOffset).limit(sourceOffset + length);
+      outDup.put(inDup);
+    }
+    // We used to return a result but disabled; return destinationOffset + length;
+  }
+
+  /**
+   * Copy from one buffer to another from given offset.
+   * <p>
+   * Note : This will advance the position marker of {@code out} but not change the position maker
+   * for {@code in}
+   * @param in source buffer
+   * @param out destination buffer
+   * @param sourceOffset offset in the source buffer
+   * @param length how many bytes to copy
+   */
+  public static void copyFromBufferToBuffer(ByteBuffer in, ByteBuffer out, int sourceOffset,
+      int length) {
+    if (in.hasArray() && out.hasArray()) {
+      System.arraycopy(in.array(), sourceOffset + in.arrayOffset(), out.array(), out.position()
+          + out.arrayOffset(), length);
+      skip(out, length);
+    } else if (UNSAFE_AVAIL) {
+      UnsafeAccess.copy(in, sourceOffset, out, out.position(), length);
+      skip(out, length);
+    } else {
+      ByteBuffer inDup = in.duplicate();
+      inDup.position(sourceOffset).limit(sourceOffset + length);
+      out.put(inDup);
     }
   }
 
@@ -658,6 +745,7 @@ public final class ByteBufferUtils {
    * @param limit Maximal length of common prefix.
    * @return Length of prefix.
    */
+  @SuppressWarnings("unused")
   public static int findCommonPrefix(ByteBuffer buffer, int offsetLeft,
       int offsetRight, int limit) {
     int prefix = 0;
@@ -671,7 +759,6 @@ public final class ByteBufferUtils {
     return prefix;
   }
 
-
   /**
    * Find length of common prefix in two arrays.
    * @param left Array to be compared.
@@ -791,6 +878,34 @@ public final class ByteBufferUtils {
     return output;
   }
 
+  public static boolean equals(ByteBuffer buf1, int o1, int l1, ByteBuffer buf2, int o2, int l2) {
+    if ((l1 == 0) || (l2 == 0)) {
+      // both 0 length, return true, or else false
+      return l1 == l2;
+    }
+    // Since we're often comparing adjacent sorted data,
+    // it's usual to have equal arrays except for the very last byte
+    // so check that first
+    if (toByte(buf1, o1 + l1 - 1) != toByte(buf2, o2 + l2 - 1)) return false;
+    return compareTo(buf1, o1, l1, buf2, o2, l2) == 0;
+  }
+
+  /**
+   * @param buf
+   *          ByteBuffer to hash
+   * @param offset
+   *          offset to start from
+   * @param length
+   *          length to hash
+   */
+  public static int hashCode(ByteBuffer buf, int offset, int length) {
+    int hash = 1;
+    for (int i = offset; i < offset + length; i++) {
+      hash = (31 * hash) + (int) toByte(buf, i);
+    }
+    return hash;
+  }
+
   public static int compareTo(ByteBuffer buf1, int o1, int l1, ByteBuffer buf2, int o2, int l2) {
     return ComparerHolder.BEST_COMPARER.compareTo(buf1, o1, l1, buf2, o2, l2);
   }
@@ -890,6 +1005,29 @@ public final class ByteBufferUtils {
     return ConverterHolder.BEST_CONVERTER.toInt(buffer, offset);
   }
 
+  /**
+   * Converts a ByteBuffer to an int value
+   *
+   * @param buf The ByteBuffer
+   * @param offset Offset to int value
+   * @param length Number of bytes used to store the int value.
+   * @return the int value
+   * @throws IllegalArgumentException
+   *           if there's not enough bytes left in the buffer after the given offset
+   */
+  public static int readAsInt(ByteBuffer buf, int offset, final int length) {
+    if (offset + length > buf.limit()) {
+      throw new IllegalArgumentException("offset (" + offset + ") + length (" + length
+          + ") exceed the" + " limit of the buffer: " + buf.limit());
+    }
+    int n = 0;
+    for(int i = offset; i < (offset + length); i++) {
+      n <<= 8;
+      n ^= toByte(buf, i) & 0xFF;
+    }
+    return n;
+  }
+
   /**
    * Reads a long value at the given buffer's offset.
    * @param buffer
@@ -914,6 +1052,34 @@ public final class ByteBufferUtils {
     return ConverterHolder.BEST_CONVERTER.putInt(buffer, index, val);
   }
 
+  /**
+   * Reads a double value at the given buffer's offset.
+   * @param buffer
+   * @param offset offset where double is
+   * @return double value at offset
+   */
+  public static double toDouble(ByteBuffer buffer, int offset) {
+    return Double.longBitsToDouble(toLong(buffer, offset));
+  }
+
+  /**
+   * Reads a BigDecimal value at the given buffer's offset.
+   * @param buffer
+   * @param offset
+   * @return BigDecimal value at offset
+   */
+  public static BigDecimal toBigDecimal(ByteBuffer buffer, int offset, int length) {
+    if (buffer == null || length < Bytes.SIZEOF_INT + 1 ||
+      (offset + length > buffer.limit())) {
+      return null;
+    }
+
+    int scale = toInt(buffer, offset);
+    byte[] tcBytes = new byte[length - Bytes.SIZEOF_INT];
+    copyFromBufferToArray(tcBytes, buffer, offset + Bytes.SIZEOF_INT, 0, length - Bytes.SIZEOF_INT);
+    return new BigDecimal(new BigInteger(tcBytes), scale);
+  }
+
   /**
    * Put a short value out to the given ByteBuffer's current position in big-endian format.
    * This also advances the position in buffer by short size.
@@ -928,6 +1094,13 @@ public final class ByteBufferUtils {
     return ConverterHolder.BEST_CONVERTER.putShort(buffer, index, val);
   }
 
+  public static int putAsShort(ByteBuffer buf, int index, int val) {
+    buf.put(index + 1, (byte) val);
+    val >>= 8;
+    buf.put(index, (byte) val);
+    return index + Bytes.SIZEOF_SHORT;
+  }
+
   /**
    * Put a long value out to the given ByteBuffer's current position in big-endian format.
    * This also advances the position in buffer by long size.
@@ -942,6 +1115,28 @@ public final class ByteBufferUtils {
     return ConverterHolder.BEST_CONVERTER.putLong(buffer, index, val);
   }
 
+  /**
+   * Copies the bytes from given array's offset to length part into the given buffer. Puts the bytes
+   * to buffer's current position. This also advances the position in the 'out' buffer by 'length'
+   * @param out
+   * @param in
+   * @param inOffset
+   * @param length
+   */
+  public static void copyFromArrayToBuffer(ByteBuffer out, byte[] in, int inOffset, int length) {
+    if (out.hasArray()) {
+      System.arraycopy(in, inOffset, out.array(), out.arrayOffset() + out.position(), length);
+      // Move the position in out by length
+      out.position(out.position() + length);
+    } else if (UNSAFE_AVAIL) {
+      UnsafeAccess.copy(in, inOffset, out, out.position(), length);
+      // Move the position in out by length
+      out.position(out.position() + length);
+    } else {
+      out.put(in, inOffset, length);
+    }
+  }
+
   /**
    * Copies bytes from given array's offset to length part into the given buffer. Puts the bytes
    * to buffer's given position. This doesn't affact the position of buffer.
@@ -984,4 +1179,43 @@ public final class ByteBufferUtils {
       inDup.get(out, destinationOffset, length);
     }
   }
+
+  /**
+   * Similar to  {@link Arrays#copyOfRange(byte[], int, int)}
+   * @param original the buffer from which the copy has to happen
+   * @param from the starting index
+   * @param to the ending index
+   * @return a byte[] created out of the copy
+   */
+  public static byte[] copyOfRange(ByteBuffer original, int from, int to) {
+    int newLength = to - from;
+    if (newLength < 0) throw new IllegalArgumentException(from + " > " + to);
+    byte[] copy = new byte[newLength];
+    ByteBufferUtils.copyFromBufferToArray(copy, original, from, 0, newLength);
+    return copy;
+  }
+
+  // For testing purpose
+  public static String toStringBinary(final ByteBuffer b, int off, int len) {
+    StringBuilder result = new StringBuilder();
+    // Just in case we are passed a 'len' that is > buffer length...
+    if (off >= b.capacity())
+      return result.toString();
+    if (off + len > b.capacity())
+      len = b.capacity() - off;
+    for (int i = off; i < off + len; ++i) {
+      int ch = b.get(i) & 0xFF;
+      if ((ch >= '0' && ch <= '9') || (ch >= 'A' && ch <= 'Z') || (ch >= 'a' && ch <= 'z')
+          || " `~!@#$%^&*()-_=+[]{}|;:'\",.<>/?".indexOf(ch) >= 0) {
+        result.append((char) ch);
+      } else {
+        result.append(String.format("\\x%02X", ch));
+      }
+    }
+    return result.toString();
+  }
+
+  public static String toStringBinary(final ByteBuffer b) {
+    return toStringBinary(b, 0, b.capacity());
+  }
 }
