@@ -17,28 +17,38 @@
  */
 package org.apache.hadoop.hbase;
 
-
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertTrue;
 
 import java.util.concurrent.TimeUnit;
-
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
 import org.apache.hadoop.hbase.TestChoreService.ScheduledChoreSamples.CountingChore;
 import org.apache.hadoop.hbase.TestChoreService.ScheduledChoreSamples.DoNothingChore;
 import org.apache.hadoop.hbase.TestChoreService.ScheduledChoreSamples.FailInitialChore;
 import org.apache.hadoop.hbase.TestChoreService.ScheduledChoreSamples.SampleStopper;
 import org.apache.hadoop.hbase.TestChoreService.ScheduledChoreSamples.SleepingChore;
 import org.apache.hadoop.hbase.TestChoreService.ScheduledChoreSamples.SlowChore;
-import org.apache.hadoop.hbase.testclassification.SmallTests;
+import org.apache.hadoop.hbase.testclassification.MediumTests;
+import org.apache.hadoop.hbase.util.Threads;
+import org.junit.ClassRule;
+import org.junit.Rule;
 import org.junit.Test;
 import org.junit.experimental.categories.Category;
+import org.junit.rules.TestName;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
-@Category(SmallTests.class)
+@Category(MediumTests.class)
 public class TestChoreService {
-  public static final Log log = LogFactory.getLog(TestChoreService.class);
+
+  @ClassRule
+  public static final HBaseClassTestRule CLASS_RULE =
+      HBaseClassTestRule.forClass(TestChoreService.class);
+
+  public static final Logger log = LoggerFactory.getLogger(TestChoreService.class);
+
+  @Rule
+  public TestName name = new TestName();
 
   /**
    * A few ScheduledChore samples that are useful for testing with ChoreService
@@ -167,14 +177,18 @@ public class TestChoreService {
       @Override
       protected boolean initialChore() {
         countOfChoreCalls++;
-        if (outputOnTicks) outputTickCount();
+        if (outputOnTicks) {
+          outputTickCount();
+        }
         return true;
       }
 
       @Override
       protected void chore() {
         countOfChoreCalls++;
-        if (outputOnTicks) outputTickCount();
+        if (outputOnTicks) {
+          outputTickCount();
+        }
       }
 
       private void outputTickCount() {
@@ -235,7 +249,7 @@ public class TestChoreService {
     }
   }
 
-  @Test (timeout=20000)
+  @Test
   public void testInitialChorePrecedence() throws InterruptedException {
     ChoreService service = new ChoreService("testInitialChorePrecedence");
 
@@ -249,22 +263,22 @@ public class TestChoreService {
       int loopCount = 0;
       boolean brokeOutOfLoop = false;
 
-     while (!chore.isInitialChoreComplete() && chore.isScheduled()) {
-       Thread.sleep(failureThreshold * period);
-       loopCount++;
-       if (loopCount > 3) {
-         brokeOutOfLoop = true;
-         break;
-       }
-    }
+      while (!chore.isInitialChoreComplete() && chore.isScheduled()) {
+        Thread.sleep(failureThreshold * period);
+        loopCount++;
+        if (loopCount > 3) {
+          brokeOutOfLoop = true;
+          break;
+        }
+      }
 
-    assertFalse(brokeOutOfLoop);
+      assertFalse(brokeOutOfLoop);
     } finally {
       shutdownService(service);
     }
   }
 
-  @Test (timeout=20000)
+  @Test
   public void testCancelChore() throws InterruptedException {
     final int period = 100;
     ScheduledChore chore1 = new DoNothingChore("chore1", period);
@@ -281,7 +295,7 @@ public class TestChoreService {
     }
   }
 
-  @Test (timeout=20000)
+  @Test
   public void testScheduledChoreConstruction() {
     final String NAME = "chore";
     final int PERIOD = 100;
@@ -314,12 +328,13 @@ public class TestChoreService {
       invalidDelayChore.getInitialDelay());
   }
 
-  @Test (timeout=20000)
+  @Test
   public void testChoreServiceConstruction() throws InterruptedException {
     final int corePoolSize = 10;
     final int defaultCorePoolSize = ChoreService.MIN_CORE_POOL_SIZE;
 
-    ChoreService customInit = new ChoreService("testChoreServiceConstruction_custom", corePoolSize, false);
+    ChoreService customInit =
+        new ChoreService("testChoreServiceConstruction_custom", corePoolSize, false);
     try {
       assertEquals(corePoolSize, customInit.getCorePoolSize());
     } finally {
@@ -341,7 +356,7 @@ public class TestChoreService {
     }
   }
 
-  @Test (timeout=20000)
+  @Test
   public void testFrequencyOfChores() throws InterruptedException {
     final int period = 100;
     // Small delta that acts as time buffer (allowing chores to complete if running slowly)
@@ -368,7 +383,7 @@ public class TestChoreService {
     }
   }
 
-  @Test (timeout=20000)
+  @Test
   public void testForceTrigger() throws InterruptedException {
     final int period = 100;
     final int delta = period/10;
@@ -406,7 +421,7 @@ public class TestChoreService {
     }
   }
 
-  @Test (timeout=20000)
+  @Test
   public void testCorePoolIncrease() throws InterruptedException {
     final int initialCorePoolSize = 3;
     ChoreService service = new ChoreService("testCorePoolIncrease", initialCorePoolSize, false);
@@ -446,7 +461,7 @@ public class TestChoreService {
     }
   }
 
-  @Test(timeout = 30000)
+  @Test
   public void testCorePoolDecrease() throws InterruptedException {
     final int initialCorePoolSize = 3;
     ChoreService service = new ChoreService("testCorePoolDecrease", initialCorePoolSize, false);
@@ -515,7 +530,7 @@ public class TestChoreService {
     }
   }
 
-  @Test (timeout=20000)
+  @Test
   public void testNumberOfRunningChores() throws InterruptedException {
     ChoreService service = new ChoreService("testNumberOfRunningChores");
 
@@ -556,12 +571,12 @@ public class TestChoreService {
     }
   }
 
-  @Test (timeout=20000)
+  @Test
   public void testNumberOfChoresMissingStartTime() throws InterruptedException {
     ChoreService service = new ChoreService("testNumberOfChoresMissingStartTime");
 
     final int period = 100;
-    final int sleepTime = 5 * period;
+    final int sleepTime = 20 * period;
 
     try {
       // Slow chores sleep for a length of time LONGER than their period. Thus, SlowChores
@@ -604,7 +619,7 @@ public class TestChoreService {
    * been scheduled with the service. For example, if 4 ScheduledChores are scheduled with a
    * ChoreService, the number of threads in the ChoreService's core pool should never exceed 4
    */
-  @Test (timeout=20000)
+  @Test
   public void testMaximumChoreServiceThreads() throws InterruptedException {
     ChoreService service = new ChoreService("testMaximumChoreServiceThreads");
 
@@ -651,7 +666,7 @@ public class TestChoreService {
     }
   }
 
-  @Test (timeout=20000)
+  @Test
   public void testChangingChoreServices() throws InterruptedException {
     final int period = 100;
     final int sleepTime = 10;
@@ -690,7 +705,7 @@ public class TestChoreService {
     }
   }
 
-  @Test (timeout=20000)
+  @Test
   public void testStopperForScheduledChores() throws InterruptedException {
     ChoreService service = new ChoreService("testStopperForScheduledChores");
     Stoppable stopperForGroup1 = new SampleStopper();
@@ -745,7 +760,7 @@ public class TestChoreService {
     }
   }
 
-  @Test (timeout=20000)
+  @Test
   public void testShutdownCancelsScheduledChores() throws InterruptedException {
     final int period = 100;
     ChoreService service = new ChoreService("testShutdownCancelsScheduledChores");
@@ -769,7 +784,7 @@ public class TestChoreService {
     assertFalse(successChore3.isScheduled());
   }
 
-  @Test (timeout=20000)
+  @Test
   public void testShutdownWorksWhileChoresAreExecuting() throws InterruptedException {
     final int period = 100;
     final int sleep = 5 * period;
@@ -797,7 +812,7 @@ public class TestChoreService {
     }
   }
 
-  @Test (timeout=20000)
+  @Test
   public void testShutdownRejectsNewSchedules() throws InterruptedException {
     final int period = 100;
     ChoreService service = new ChoreService("testShutdownRejectsNewSchedules");
@@ -826,4 +841,21 @@ public class TestChoreService {
     assertFalse(service.scheduleChore(failChore3));
     assertFalse(failChore3.isScheduled());
   }
+
+  /**
+   * for HBASE-25014
+   */
+  @Test(timeout = 10000)
+  public void testInitialDelay() {
+    ChoreService service = new ChoreService(name.getMethodName());
+    SampleStopper stopper = new SampleStopper();
+    service.scheduleChore(new ScheduledChore("chore", stopper, 1000, 2000) {
+      @Override protected void chore() {
+        stopper.stop("test");
+      }
+    });
+    while (!stopper.isStopped()) {
+      Threads.sleep(1000);
+    }
+  }
 }
