@@ -17,7 +17,8 @@
  */
 package org.apache.hadoop.hbase;
 
-import static org.junit.Assert.assertArrayEquals;
+import static org.hamcrest.MatcherAssert.assertThat;
+import static org.hamcrest.Matchers.arrayContainingInAnyOrder;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertTrue;
@@ -36,23 +37,27 @@ import java.util.jar.Attributes;
 import java.util.jar.JarEntry;
 import java.util.jar.JarOutputStream;
 import java.util.jar.Manifest;
-
 import javax.tools.JavaCompiler;
 import javax.tools.ToolProvider;
-
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.apache.hadoop.hbase.testclassification.MiscTests;
 import org.apache.hadoop.hbase.testclassification.SmallTests;
 import org.junit.AfterClass;
 import org.junit.BeforeClass;
+import org.junit.ClassRule;
 import org.junit.Rule;
 import org.junit.Test;
 import org.junit.experimental.categories.Category;
 import org.junit.rules.TestName;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
-@Category(SmallTests.class)
+@Category({MiscTests.class, SmallTests.class})
 public class TestClassFinder {
-  private static final Log LOG = LogFactory.getLog(TestClassFinder.class);
+  @ClassRule
+  public static final HBaseClassTestRule CLASS_RULE =
+      HBaseClassTestRule.forClass(TestClassFinder.class);
+
+  private static final Logger LOG = LoggerFactory.getLogger(TestClassFinder.class);
 
   @Rule public TestName name = new TestName();
   private static final HBaseCommonTestingUtility testUtil = new HBaseCommonTestingUtility();
@@ -148,12 +153,8 @@ public class TestClassFinder {
     final String classNamePrefix = name.getMethodName();
     LOG.info("Created jar " + createAndLoadJar("", classNamePrefix, counter));
 
-    ClassFinder.FileNameFilter notExcNameFilter = new ClassFinder.FileNameFilter() {
-      @Override
-      public boolean isCandidateFile(String fileName, String absFilePath) {
-        return !fileName.startsWith(PREFIX);
-      }
-    };
+    ClassFinder.FileNameFilter notExcNameFilter =
+      (fileName, absFilePath) -> !fileName.startsWith(PREFIX);
     ClassFinder incClassesFinder = new ClassFinder(null, notExcNameFilter, null, classLoader);
     Set<Class<?>> incClasses = incClassesFinder.findClasses(
         makePackageName("", counter), false);
@@ -168,12 +169,7 @@ public class TestClassFinder {
     final String classNamePrefix = name.getMethodName();
     LOG.info("Created jar " + createAndLoadJar("", classNamePrefix, counter));
 
-    final ClassFinder.ClassFilter notExcClassFilter = new ClassFinder.ClassFilter() {
-      @Override
-      public boolean isCandidateClass(Class<?> c) {
-        return !c.getSimpleName().startsWith(PREFIX);
-      }
-    };
+    final ClassFinder.ClassFilter notExcClassFilter = c -> !c.getSimpleName().startsWith(PREFIX);
     ClassFinder incClassesFinder = new ClassFinder(null, null, notExcClassFilter, classLoader);
     Set<Class<?>> incClasses = incClassesFinder.findClasses(
         makePackageName("", counter), false);
@@ -206,12 +202,7 @@ public class TestClassFinder {
       new File(excludedJar).toURI().getRawSchemeSpecificPart();
 
     final ClassFinder.ResourcePathFilter notExcJarFilter =
-        new ClassFinder.ResourcePathFilter() {
-      @Override
-      public boolean isCandidatePath(String resourcePath, boolean isJar) {
-        return !isJar || !resourcePath.equals(excludedJarResource);
-      }
-    };
+      (resourcePath, isJar) -> !isJar || !resourcePath.equals(excludedJarResource);
     ClassFinder incClassesFinder = new ClassFinder(notExcJarFilter, null, null, classLoader);
     Set<Class<?>> incClasses = incClassesFinder.findClasses(
         makePackageName("", counter), false);
@@ -254,12 +245,8 @@ public class TestClassFinder {
     String pkgNameSuffix = name.getMethodName();
     LOG.info("Created jar " + createAndLoadJar(pkgNameSuffix, classNamePrefix, counter));
     final String classNameToFilterOut = classNamePrefix + counter;
-    final ClassFinder.FileNameFilter notThisFilter = new ClassFinder.FileNameFilter() {
-      @Override
-      public boolean isCandidateFile(String fileName, String absFilePath) {
-        return !fileName.equals(classNameToFilterOut + ".class");
-      }
-    };
+    final ClassFinder.FileNameFilter notThisFilter =
+      (fileName, absFilePath) -> !fileName.equals(classNameToFilterOut + ".class");
     String pkgName = makePackageName(pkgNameSuffix, counter);
     ClassFinder allClassesFinder = new ClassFinder(classLoader);
     Set<Class<?>> allClasses = allClassesFinder.findClasses(pkgName, false);
@@ -279,12 +266,7 @@ public class TestClassFinder {
     String pkgNameSuffix = name.getMethodName();
     LOG.info("Created jar " + createAndLoadJar(pkgNameSuffix, classNamePrefix, counter));
     final Class<?> clazz = makeClass(pkgNameSuffix, classNamePrefix, counter);
-    final ClassFinder.ClassFilter notThisFilter = new ClassFinder.ClassFilter() {
-      @Override
-      public boolean isCandidateClass(Class<?> c) {
-        return c != clazz;
-      }
-    };
+    final ClassFinder.ClassFilter notThisFilter = c -> c != clazz;
     String pkgName = makePackageName(pkgNameSuffix, counter);
     ClassFinder allClassesFinder = new ClassFinder(classLoader);
     Set<Class<?>> allClasses = allClassesFinder.findClasses(pkgName, false);
@@ -298,12 +280,8 @@ public class TestClassFinder {
   @Test
   public void testClassFinderFiltersByPathInDirs() throws Exception {
     final String hardcodedThisSubdir = "hbase-common";
-    final ClassFinder.ResourcePathFilter notExcJarFilter = new ClassFinder.ResourcePathFilter() {
-      @Override
-      public boolean isCandidatePath(String resourcePath, boolean isJar) {
-        return isJar || !resourcePath.contains(hardcodedThisSubdir);
-      }
-    };
+    final ClassFinder.ResourcePathFilter notExcJarFilter =
+      (resourcePath, isJar) -> isJar || !resourcePath.contains(hardcodedThisSubdir);
     String thisPackage = this.getClass().getPackage().getName();
     ClassFinder notThisClassFinder = new ClassFinder(notExcJarFilter, null, null, classLoader);
     Set<Class<?>> notAllClasses = notThisClassFinder.findClasses(thisPackage, false);
@@ -318,7 +296,10 @@ public class TestClassFinder {
     Set<Class<?>> pkgClasses = allClassesFinder.findClasses(
         ClassFinder.class.getPackage().getName(), false);
     Set<Class<?>> defaultClasses = allClassesFinder.findClasses(false);
-    assertArrayEquals(pkgClasses.toArray(), defaultClasses.toArray());
+    Object[] pkgClassesArray = pkgClasses.toArray();
+    Object[] defaultClassesArray = defaultClasses.toArray();
+    assertEquals(pkgClassesArray.length, defaultClassesArray.length);
+    assertThat(pkgClassesArray, arrayContainingInAnyOrder(defaultClassesArray));
   }
 
   private static class FileAndPath {
