@@ -1,6 +1,4 @@
 /**
- * Copyright The Apache Software Foundation
- *
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -19,47 +17,53 @@
  */
 package org.apache.hadoop.hbase.rsgroup;
 
-import com.google.common.collect.Maps;
-import com.google.common.collect.Sets;
+import static org.junit.Assert.assertTrue;
 
 import java.io.IOException;
+import java.util.ArrayList;
+import java.util.EnumSet;
 import java.util.HashSet;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
+import java.util.Optional;
 import java.util.Random;
 import java.util.Set;
 import java.util.TreeMap;
-
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
-import org.apache.hadoop.hbase.ClusterStatus;
+import java.util.regex.Pattern;
+import org.apache.hadoop.hbase.ClusterMetrics;
+import org.apache.hadoop.hbase.ClusterMetrics.Option;
 import org.apache.hadoop.hbase.HBaseCluster;
 import org.apache.hadoop.hbase.HBaseTestingUtility;
 import org.apache.hadoop.hbase.HConstants;
-import org.apache.hadoop.hbase.HRegionInfo;
-import org.apache.hadoop.hbase.HTableDescriptor;
 import org.apache.hadoop.hbase.MiniHBaseCluster;
 import org.apache.hadoop.hbase.NamespaceDescriptor;
-import org.apache.hadoop.hbase.RegionLoad;
 import org.apache.hadoop.hbase.ServerName;
 import org.apache.hadoop.hbase.TableName;
 import org.apache.hadoop.hbase.Waiter;
-import org.apache.hadoop.hbase.client.HBaseAdmin;
-import org.apache.hadoop.hbase.coprocessor.BaseMasterObserver;
+import org.apache.hadoop.hbase.client.Admin;
+import org.apache.hadoop.hbase.client.RegionInfo;
+import org.apache.hadoop.hbase.client.TableDescriptor;
 import org.apache.hadoop.hbase.coprocessor.CoprocessorHost;
+import org.apache.hadoop.hbase.coprocessor.MasterCoprocessor;
 import org.apache.hadoop.hbase.coprocessor.MasterCoprocessorEnvironment;
+import org.apache.hadoop.hbase.coprocessor.MasterObserver;
 import org.apache.hadoop.hbase.coprocessor.ObserverContext;
 import org.apache.hadoop.hbase.master.HMaster;
 import org.apache.hadoop.hbase.master.MasterCoprocessorHost;
 import org.apache.hadoop.hbase.master.ServerManager;
 import org.apache.hadoop.hbase.master.snapshot.SnapshotManager;
 import org.apache.hadoop.hbase.net.Address;
-import org.apache.hadoop.hbase.util.Bytes;
-import org.apache.hadoop.hbase.util.JVMClusterUtil;
+import org.junit.Rule;
+import org.junit.rules.TestName;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
+import org.apache.hbase.thirdparty.com.google.common.collect.Maps;
+import org.apache.hbase.thirdparty.com.google.common.collect.Sets;
 
 public abstract class TestRSGroupsBase {
-  protected static final Log LOG = LogFactory.getLog(TestRSGroupsBase.class);
+  protected static final Logger LOG = LoggerFactory.getLogger(TestRSGroupsBase.class);
 
   //shared
   protected final static String groupPrefix = "Group";
@@ -68,16 +72,22 @@ public abstract class TestRSGroupsBase {
 
   //shared, cluster type specific
   protected static HBaseTestingUtility TEST_UTIL;
-  protected static HBaseAdmin admin;
+  protected static Admin admin;
   protected static HBaseCluster cluster;
   protected static RSGroupAdmin rsGroupAdmin;
   protected static HMaster master;
-  protected static boolean init = false;
-  protected static RSGroupAdminEndpoint RSGroupAdminEndpoint;
+  protected boolean INIT = false;
+  protected static RSGroupAdminEndpoint rsGroupAdminEndpoint;
   protected static CPMasterObserver observer;
 
-  public final static long WAIT_TIMEOUT = 60000*5;
+  public final static long WAIT_TIMEOUT = 60000;
   public final static int NUM_SLAVES_BASE = 4; //number of slaves for the smallest cluster
+  public static int NUM_DEAD_SERVERS = 0;
+
+  // Per test variables
+  @Rule
+  public TestName name = new TestName();
+  protected TableName tableName;
 
   public static void setUpTestBeforeClass() throws Exception {
     TEST_UTIL = new HBaseTestingUtility();
@@ -88,9 +98,6 @@ public abstract class TestRSGroupsBase {
         RSGroupBasedLoadBalancer.class.getName());
     TEST_UTIL.getConfiguration().set(CoprocessorHost.MASTER_COPROCESSOR_CONF_KEY,
         RSGroupAdminEndpoint.class.getName() + "," + CPMasterObserver.class.getName());
-    TEST_UTIL.getConfiguration().setBoolean(
-        HConstants.ZOOKEEPER_USEMULTI,
-        true);
     TEST_UTIL.startMiniCluster(NUM_SLAVES_BASE - 1);
     TEST_UTIL.getConfiguration().setInt(
         ServerManager.WAIT_ON_REGIONSERVERS_MINTOSTART,
@@ -100,10 +107,9 @@ public abstract class TestRSGroupsBase {
   }
 
   protected static void initialize() throws Exception {
-    admin = TEST_UTIL.getHBaseAdmin();
+    admin = TEST_UTIL.getAdmin();
     cluster = TEST_UTIL.getHBaseCluster();
-    master = ((MiniHBaseCluster)cluster).getMaster();
-    master.balanceSwitch(true);
+    master = TEST_UTIL.getMiniHBaseCluster().getMaster();
 
     //wait for balancer to come online
     TEST_UTIL.waitFor(WAIT_TIMEOUT, new Waiter.Predicate<Exception>() {
@@ -113,13 +119,13 @@ public abstract class TestRSGroupsBase {
             ((RSGroupBasedLoadBalancer) master.getLoadBalancer()).isOnline();
       }
     });
-    admin.setBalancerRunning(false,true);
-    rsGroupAdmin = new VerifyingRSGroupAdminClient(new RSGroupAdminClient(TEST_UTIL.getConnection()),
-        TEST_UTIL.getConfiguration());
+    admin.balancerSwitch(false, true);
+    rsGroupAdmin = new VerifyingRSGroupAdminClient(
+        new RSGroupAdminClient(TEST_UTIL.getConnection()), TEST_UTIL.getConfiguration());
     MasterCoprocessorHost host = master.getMasterCoprocessorHost();
     observer = (CPMasterObserver) host.findCoprocessor(CPMasterObserver.class.getName());
-    RSGroupAdminEndpoint =
-        host.findCoprocessors(RSGroupAdminEndpoint.class).get(0);
+    rsGroupAdminEndpoint = (RSGroupAdminEndpoint)
+        host.findCoprocessor(RSGroupAdminEndpoint.class.getName());
   }
 
   public static void tearDownAfterClass() throws Exception {
@@ -127,8 +133,10 @@ public abstract class TestRSGroupsBase {
   }
 
   public void setUpBeforeMethod() throws Exception {
-    if(!init) {
-      init = true;
+    LOG.info(name.getMethodName());
+    tableName = TableName.valueOf(tablePrefix + "_" + name.getMethodName());
+    if (!INIT) {
+      INIT = true;
       tearDownAfterMethod();
     }
     observer.resetFlags();
@@ -139,6 +147,11 @@ public abstract class TestRSGroupsBase {
     deleteNamespaceIfNecessary();
     deleteGroups();
 
+    for(ServerName sn : admin.listDecommissionedRegionServers()){
+      admin.recommissionRegionServer(sn, null);
+    }
+    assertTrue(admin.listDecommissionedRegionServers().isEmpty());
+
     int missing = NUM_SLAVES_BASE - getNumServers();
     LOG.info("Restoring servers: "+missing);
     for(int i=0; i<missing; i++) {
@@ -150,13 +163,11 @@ public abstract class TestRSGroupsBase {
         ((MiniHBaseCluster)cluster).getMaster().getServerName();
 
     try {
-      rsGroupAdmin.moveServers(
-          Sets.newHashSet(masterServerName.getAddress()),
-          "master");
+      rsGroupAdmin.moveServers(Sets.newHashSet(masterServerName.getAddress()), "master");
     } catch (Exception ex) {
-      // ignore
+      LOG.warn("Got this on setup, FYI", ex);
     }
-
+    assertTrue(observer.preMoveServersCalled);
     TEST_UTIL.waitFor(WAIT_TIMEOUT, new Waiter.Predicate<Exception>() {
       @Override
       public boolean evaluate() throws Exception {
@@ -170,52 +181,38 @@ public abstract class TestRSGroupsBase {
     });
   }
 
-  // return the real number of region servers, excluding the master embedded region server in 2.0+
-  protected int getNumServers() throws IOException {
-    ClusterStatus status = admin.getClusterStatus();
-    ServerName master = status.getMaster();
-    int count = 0;
-    for (ServerName sn : status.getServers()) {
-      if (!sn.equals(master)) {
-        count++;
-      }
-    }
-    return count;
-  }
-
-  protected RSGroupInfo addGroup(RSGroupAdmin gAdmin, String groupName,
-                                 int serverCount) throws IOException, InterruptedException {
-    RSGroupInfo defaultInfo = gAdmin
-        .getRSGroupInfo(RSGroupInfo.DEFAULT_GROUP);
-    gAdmin.addRSGroup(groupName);
-
-    Set<Address> set = new HashSet<Address>();
+  protected RSGroupInfo addGroup(String groupName, int serverCount)
+      throws IOException, InterruptedException {
+    RSGroupInfo defaultInfo = rsGroupAdmin.getRSGroupInfo(RSGroupInfo.DEFAULT_GROUP);
+    rsGroupAdmin.addRSGroup(groupName);
+    Set<Address> set = new HashSet<>();
     for(Address server: defaultInfo.getServers()) {
       if(set.size() == serverCount) {
         break;
       }
       set.add(server);
     }
-    gAdmin.moveServers(set, groupName);
-    RSGroupInfo result = gAdmin.getRSGroupInfo(groupName);
+    rsGroupAdmin.moveServers(set, groupName);
+    RSGroupInfo result = rsGroupAdmin.getRSGroupInfo(groupName);
     return result;
   }
 
-  static void removeGroup(RSGroupAdminClient groupAdmin, String groupName) throws IOException {
-    RSGroupInfo info = groupAdmin.getRSGroupInfo(groupName);
-    groupAdmin.moveTables(info.getTables(), RSGroupInfo.DEFAULT_GROUP);
-    groupAdmin.moveServers(info.getServers(), RSGroupInfo.DEFAULT_GROUP);
-    groupAdmin.removeRSGroup(groupName);
+  protected void removeGroup(String groupName) throws IOException {
+    RSGroupInfo groupInfo = rsGroupAdmin.getRSGroupInfo(groupName);
+    rsGroupAdmin.moveTables(groupInfo.getTables(), RSGroupInfo.DEFAULT_GROUP);
+    rsGroupAdmin.moveServers(groupInfo.getServers(), RSGroupInfo.DEFAULT_GROUP);
+    rsGroupAdmin.removeRSGroup(groupName);
   }
 
   protected void deleteTableIfNecessary() throws IOException {
-    for (HTableDescriptor desc : TEST_UTIL.getHBaseAdmin().listTables(tablePrefix+".*")) {
+    for (TableDescriptor desc : TEST_UTIL.getAdmin()
+      .listTableDescriptors(Pattern.compile(tablePrefix + ".*"))) {
       TEST_UTIL.deleteTable(desc.getTableName());
     }
   }
 
   protected void deleteNamespaceIfNecessary() throws IOException {
-    for (NamespaceDescriptor desc : TEST_UTIL.getHBaseAdmin().listNamespaceDescriptors()) {
+    for (NamespaceDescriptor desc : TEST_UTIL.getAdmin().listNamespaceDescriptors()) {
       if(desc.getName().startsWith(tablePrefix)) {
         admin.deleteNamespace(desc.getName());
       }
@@ -223,24 +220,23 @@ public abstract class TestRSGroupsBase {
   }
 
   protected void deleteGroups() throws IOException {
-    try (RSGroupAdmin groupAdmin = new RSGroupAdminClient(TEST_UTIL.getConnection())) {
-      for(RSGroupInfo group: groupAdmin.listRSGroups()) {
-        if(!group.getName().equals(RSGroupInfo.DEFAULT_GROUP)) {
-          groupAdmin.moveTables(group.getTables(), RSGroupInfo.DEFAULT_GROUP);
-          groupAdmin.moveServers(group.getServers(), RSGroupInfo.DEFAULT_GROUP);
-          groupAdmin.removeRSGroup(group.getName());
-        }
+    RSGroupAdmin groupAdmin = new RSGroupAdminClient(TEST_UTIL.getConnection());
+    for(RSGroupInfo group: groupAdmin.listRSGroups()) {
+      if(!group.getName().equals(RSGroupInfo.DEFAULT_GROUP)) {
+        groupAdmin.moveTables(group.getTables(), RSGroupInfo.DEFAULT_GROUP);
+        groupAdmin.moveServers(group.getServers(), RSGroupInfo.DEFAULT_GROUP);
+        groupAdmin.removeRSGroup(group.getName());
       }
     }
   }
 
-  public Map<TableName, List<String>> getTableRegionMap() throws IOException {
+  protected Map<TableName, List<String>> getTableRegionMap() throws IOException {
     Map<TableName, List<String>> map = Maps.newTreeMap();
     Map<TableName, Map<ServerName, List<String>>> tableServerRegionMap
         = getTableServerRegionMap();
     for(TableName tableName : tableServerRegionMap.keySet()) {
       if(!map.containsKey(tableName)) {
-        map.put(tableName, new LinkedList<String>());
+        map.put(tableName, new LinkedList<>());
       }
       for(List<String> subset: tableServerRegionMap.get(tableName).values()) {
         map.get(tableName).addAll(subset);
@@ -249,52 +245,51 @@ public abstract class TestRSGroupsBase {
     return map;
   }
 
-  public Map<TableName, Map<ServerName, List<String>>> getTableServerRegionMap()
-      throws IOException {
+  protected Map<TableName, Map<ServerName, List<String>>> getTableServerRegionMap()
+    throws IOException {
     Map<TableName, Map<ServerName, List<String>>> map = Maps.newTreeMap();
-    ClusterStatus status = TEST_UTIL.getHBaseClusterInterface().getClusterStatus();
-    for(ServerName serverName : status.getServers()) {
-      for(RegionLoad rl : status.getLoad(serverName).getRegionsLoad().values()) {
-        TableName tableName = null;
-        try {
-          tableName = HRegionInfo.getTable(rl.getName());
-        } catch (IllegalArgumentException e) {
-          LOG.warn("Failed parse a table name from regionname=" +
-              Bytes.toStringBinary(rl.getName()));
-          continue;
-        }
-        if(!map.containsKey(tableName)) {
-          map.put(tableName, new TreeMap<ServerName, List<String>>());
-        }
-        if(!map.get(tableName).containsKey(serverName)) {
-          map.get(tableName).put(serverName, new LinkedList<String>());
-        }
-        map.get(tableName).get(serverName).add(rl.getNameAsString());
+    Admin admin = TEST_UTIL.getAdmin();
+    ClusterMetrics metrics =
+      admin.getClusterMetrics(EnumSet.of(ClusterMetrics.Option.SERVERS_NAME));
+    for (ServerName serverName : metrics.getServersName()) {
+      for (RegionInfo region : admin.getRegions(serverName)) {
+        TableName tableName = region.getTable();
+        map.computeIfAbsent(tableName, k -> new TreeMap<>())
+          .computeIfAbsent(serverName, k -> new ArrayList<>()).add(region.getRegionNameAsString());
       }
     }
     return map;
   }
 
+  // return the real number of region servers, excluding the master embedded region server in 2.0+
+  protected int getNumServers() throws IOException {
+    ClusterMetrics status =
+        admin.getClusterMetrics(EnumSet.of(Option.MASTER, Option.LIVE_SERVERS));
+    ServerName masterName = status.getMasterName();
+    int count = 0;
+    for (ServerName sn : status.getLiveServerMetrics().keySet()) {
+      if (!sn.equals(masterName)) {
+        count++;
+      }
+    }
+    return count;
+  }
+
   protected String getGroupName(String baseName) {
-    return groupPrefix+"_"+baseName+"_"+rand.nextInt(Integer.MAX_VALUE);
+    return groupPrefix + "_" + baseName + "_" + rand.nextInt(Integer.MAX_VALUE);
   }
 
   /**
    * The server name in group does not contain the start code, this method will find out the start
    * code and construct the ServerName object.
    */
-  protected final ServerName getServerName(Address addr) {
-    for (JVMClusterUtil.RegionServerThread rsThread:
-      TEST_UTIL.getMiniHBaseCluster().getRegionServerThreads()) {
-      ServerName sn = rsThread.getRegionServer().getServerName();
-      if (sn.getAddress().equals(addr)) {
-        return sn;
-      }
-    }
-    return null;
+  protected ServerName getServerName(Address addr) {
+    return TEST_UTIL.getMiniHBaseCluster().getRegionServerThreads().stream()
+      .map(t -> t.getRegionServer().getServerName()).filter(sn -> sn.getAddress().equals(addr))
+      .findFirst().get();
   }
 
-  public static class CPMasterObserver extends BaseMasterObserver {
+  public static class CPMasterObserver implements MasterCoprocessor, MasterObserver {
     boolean preBalanceRSGroupCalled = false;
     boolean postBalanceRSGroupCalled = false;
     boolean preMoveServersCalled = false;
@@ -331,6 +326,11 @@ public abstract class TestRSGroupsBase {
       postRenameRSGroupCalled = false;
     }
 
+    @Override
+    public Optional<MasterObserver> getMasterObserver() {
+      return Optional.of(this);
+    }
+
     @Override
     public void preMoveServersAndTables(final ObserverContext<MasterCoprocessorEnvironment> ctx,
         Set<Address> servers, Set<TableName> tables, String targetGroup) throws IOException {
@@ -419,14 +419,15 @@ public abstract class TestRSGroupsBase {
 
     @Override
     public void preRenameRSGroup(ObserverContext<MasterCoprocessorEnvironment> ctx,
-                                 String oldName, String newName) throws IOException {
+        String oldName, String newName) throws IOException {
       preRenameRSGroupCalled = true;
     }
 
     @Override
     public void postRenameRSGroup(ObserverContext<MasterCoprocessorEnvironment> ctx,
-                                  String oldName, String newName) throws IOException {
+        String oldName, String newName) throws IOException {
       postRenameRSGroupCalled = true;
     }
   }
+
 }
