@@ -1,5 +1,4 @@
-/*
- *
+/**
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -16,37 +15,43 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.hadoop.hbase.client;
 
 import static org.apache.hadoop.hbase.HBaseTestCase.assertByteEquals;
+import static org.hamcrest.MatcherAssert.assertThat;
+import static org.hamcrest.Matchers.greaterThan;
+import static org.hamcrest.Matchers.is;
+import static org.hamcrest.Matchers.lessThan;
 
 import java.io.IOException;
 import java.nio.ByteBuffer;
+import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
 import java.util.NoSuchElementException;
-
 import junit.framework.TestCase;
-
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
 import org.apache.hadoop.hbase.Cell;
+import org.apache.hadoop.hbase.CellComparator;
 import org.apache.hadoop.hbase.CellScanner;
 import org.apache.hadoop.hbase.CellUtil;
-import org.apache.hadoop.hbase.HConstants;
+import org.apache.hadoop.hbase.HBaseClassTestRule;
 import org.apache.hadoop.hbase.KeyValue;
-import org.apache.hadoop.hbase.Tag;
-import org.apache.hadoop.hbase.protobuf.generated.ClientProtos;
+import org.apache.hadoop.hbase.testclassification.ClientTests;
 import org.apache.hadoop.hbase.testclassification.SmallTests;
 import org.apache.hadoop.hbase.util.Bytes;
-import org.junit.Test;
+import org.junit.ClassRule;
 import org.junit.experimental.categories.Category;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
-@Category(SmallTests.class)
+@Category({SmallTests.class, ClientTests.class})
 public class TestResult extends TestCase {
 
-  private static final Log LOG = LogFactory.getLog(TestResult.class.getName());
+  @ClassRule
+  public static final HBaseClassTestRule CLASS_RULE =
+      HBaseClassTestRule.forClass(TestResult.class);
+
+  private static final Logger LOG = LoggerFactory.getLogger(TestResult.class.getName());
 
   static KeyValue[] genKVs(final byte[] row, final byte[] family,
                            final byte[] value,
@@ -73,7 +78,7 @@ public class TestResult extends TestCase {
    */
   public void testResultAsCellScanner() throws IOException {
     Cell [] cells = genKVs(row, family, value, 1, 10);
-    Arrays.sort(cells, KeyValue.COMPARATOR);
+    Arrays.sort(cells, CellComparator.getInstance());
     Result r = Result.create(cells);
     assertSame(r, cells);
     // Assert I run over same result multiple times.
@@ -95,7 +100,7 @@ public class TestResult extends TestCase {
   public void testBasicGetColumn() throws Exception {
     KeyValue [] kvs = genKVs(row, family, value, 1, 100);
 
-    Arrays.sort(kvs, KeyValue.COMPARATOR);
+    Arrays.sort(kvs, CellComparator.getInstance());
 
     Result r = Result.create(kvs);
 
@@ -115,12 +120,14 @@ public class TestResult extends TestCase {
     assertNull(r.current());
   }
 
-  public void testAdvanceMultipleOnEmptyCell() throws IOException {
+  public void testAdvanceTwiceOnEmptyCell() throws IOException {
     Result r = Result.create(new Cell[0]);
-    // After HBASE-26688, advance of result with empty cell list will always return false.
-    // Here 10 is an arbitrary number to test the logic.
-    for (int i = 0; i < 10; i++) {
-      assertFalse(r.advance());
+    assertFalse(r.advance());
+    try {
+      r.advance();
+      fail("NoSuchElementException should have been thrown!");
+    } catch (NoSuchElementException ex) {
+      LOG.debug("As expected: " + ex.getMessage());
     }
   }
 
@@ -132,7 +139,7 @@ public class TestResult extends TestCase {
     System.arraycopy(kvs1, 0, kvs, 0, kvs1.length);
     System.arraycopy(kvs2, 0, kvs, kvs1.length, kvs2.length);
 
-    Arrays.sort(kvs, KeyValue.COMPARATOR);
+    Arrays.sort(kvs, CellComparator.getInstance());
 
     Result r = Result.create(kvs);
     for (int i = 0; i < 100; ++i) {
@@ -149,7 +156,7 @@ public class TestResult extends TestCase {
   public void testBasicGetValue() throws Exception {
     KeyValue [] kvs = genKVs(row, family, value, 1, 100);
 
-    Arrays.sort(kvs, KeyValue.COMPARATOR);
+    Arrays.sort(kvs, CellComparator.getInstance());
 
     Result r = Result.create(kvs);
 
@@ -169,7 +176,7 @@ public class TestResult extends TestCase {
     System.arraycopy(kvs1, 0, kvs, 0, kvs1.length);
     System.arraycopy(kvs2, 0, kvs, kvs1.length, kvs2.length);
 
-    Arrays.sort(kvs, KeyValue.COMPARATOR);
+    Arrays.sort(kvs, CellComparator.getInstance());
 
     Result r = Result.create(kvs);
     for (int i = 0; i < 100; ++i) {
@@ -183,7 +190,7 @@ public class TestResult extends TestCase {
   public void testBasicLoadValue() throws Exception {
     KeyValue [] kvs = genKVs(row, family, value, 1, 100);
 
-    Arrays.sort(kvs, KeyValue.COMPARATOR);
+    Arrays.sort(kvs, CellComparator.getInstance());
 
     Result r = Result.create(kvs);
     ByteBuffer loadValueBuffer = ByteBuffer.allocate(1024);
@@ -194,7 +201,7 @@ public class TestResult extends TestCase {
       loadValueBuffer.clear();
       r.loadValue(family, qf, loadValueBuffer);
       loadValueBuffer.flip();
-      assertEquals(ByteBuffer.wrap(Bytes.add(value, Bytes.toBytes(i))), loadValueBuffer);
+      assertEquals(loadValueBuffer, ByteBuffer.wrap(Bytes.add(value, Bytes.toBytes(i))));
       assertEquals(ByteBuffer.wrap(Bytes.add(value, Bytes.toBytes(i))),
           r.getValueAsByteBuffer(family, qf));
     }
@@ -208,7 +215,7 @@ public class TestResult extends TestCase {
     System.arraycopy(kvs1, 0, kvs, 0, kvs1.length);
     System.arraycopy(kvs2, 0, kvs, kvs1.length, kvs2.length);
 
-    Arrays.sort(kvs, KeyValue.COMPARATOR);
+    Arrays.sort(kvs, CellComparator.getInstance());
 
     ByteBuffer loadValueBuffer = ByteBuffer.allocate(1024);
 
@@ -219,7 +226,7 @@ public class TestResult extends TestCase {
       loadValueBuffer.clear();
       r.loadValue(family, qf, loadValueBuffer);
       loadValueBuffer.flip();
-      assertEquals(ByteBuffer.wrap(Bytes.add(value, Bytes.toBytes(i))), loadValueBuffer);
+      assertEquals(loadValueBuffer, ByteBuffer.wrap(Bytes.add(value, Bytes.toBytes(i))));
       assertEquals(ByteBuffer.wrap(Bytes.add(value, Bytes.toBytes(i))),
           r.getValueAsByteBuffer(family, qf));
     }
@@ -248,47 +255,33 @@ public class TestResult extends TestCase {
     }
   }
 
-  /**
-   *  Test Result#compareResults when tags are present.
-   * @throws Exception Exception
-   */
-  @Test
-  public void testCompareResultsWithTags() throws Exception {
-    byte[] qualifier = Bytes.toBytes("q");
-    String tagValue1 = "tagV1";
-    String tagValue2 = "tagV2";
-    // Create cell with 2 tags
-    KeyValue kv1 = new KeyValue(row, family, qualifier, HConstants.LATEST_TIMESTAMP,
-      value, new Tag[] {new Tag((byte) 1, tagValue1), new Tag((byte) 2, tagValue2)});
-
-    // Create cell with no tags.
-    KeyValue kv2 = new KeyValue(row, family, qualifier, HConstants.LATEST_TIMESTAMP,
-      value);
-
-    Result r1 = Result.create(new KeyValue[] {kv1});
-    Result r2 = Result.create(new KeyValue[] {kv2});
-    // Compare result with self.
-    Result.compareResults(r1, r1);
-
-    try {
-      // Compare result with 2 tags v/s no tags.
-      Result.compareResults(r1, r2);
-      fail();
-    } catch (Exception e) {
-      assertTrue(e.getMessage().startsWith("This result was different:"));
+  public void testCompareResultMemoryUsage() {
+    List<Cell> cells1 = new ArrayList<>();
+    for (long i = 0; i < 100; i++) {
+      cells1.add(new KeyValue(row, family, Bytes.toBytes(i), value));
     }
 
-    // Create cell with just 1 tag.
-    String tagValue3 = "tagV3";
-    KeyValue kv3 = new KeyValue(row, family, qualifier, HConstants.LATEST_TIMESTAMP,
-      value, new Tag[] {new Tag((byte) 1, tagValue3)});
-    Result r3 = Result.create(new KeyValue[] {kv3});
+    List<Cell> cells2 = new ArrayList<>();
+    for (long i = 0; i < 100; i++) {
+      cells2.add(new KeyValue(row, family, Bytes.toBytes(i), Bytes.toBytes(i)));
+    }
+
+    Result r1 = Result.create(cells1);
+    Result r2 = Result.create(cells2);
     try {
-      // Compare result with 2 tags v/s 1 tag.
-      Result.compareResults(r1, r3);
+      Result.compareResults(r1, r2);
       fail();
-    } catch (Exception e) {
-      assertTrue(e.getMessage().startsWith("This result was different:"));
+    } catch (Exception x) {
+      assertTrue(x.getMessage().startsWith("This result was different:"));
+      assertThat(x.getMessage().length(), is(greaterThan(100)));
+    }
+
+    try {
+      Result.compareResults(r1, r2, false);
+      fail();
+    } catch (Exception x) {
+      assertEquals("This result was different: row=row", x.getMessage());
+      assertThat(x.getMessage().length(), is(lessThan(100)));
     }
   }
 
@@ -305,12 +298,6 @@ public class TestResult extends TestCase {
     } catch (UnsupportedOperationException ex) {
       LOG.debug("As expected: " + ex.getMessage());
     }
-    try {
-      emptyResult.addResults(ClientProtos.RegionLoadStats.getDefaultInstance());
-      fail("UnsupportedOperationException should have been thrown!");
-    } catch (UnsupportedOperationException ex) {
-      LOG.debug("As expected: " + ex.getMessage());
-    }
     try {
       emptyResult.setExists(true);
       fail("UnsupportedOperationException should have been thrown!");
@@ -341,7 +328,7 @@ public class TestResult extends TestCase {
 
     KeyValue [] kvs = genKVs(Bytes.toBytes(rowSB.toString()), family,
         Bytes.toBytes(valueSB.toString()), 1, n);
-    Arrays.sort(kvs, KeyValue.COMPARATOR);
+    Arrays.sort(kvs, CellComparator.getInstance());
     ByteBuffer loadValueBuffer = ByteBuffer.allocate(1024);
     Result r = Result.create(kvs);
 
