@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more contributor license
  * agreements. See the NOTICE file distributed with this work for additional information regarding
  * copyright ownership. The ASF licenses this file to you under the Apache License, Version 2.0 (the
@@ -17,7 +17,6 @@ package org.apache.hadoop.hbase.master.balancer;
 import static junit.framework.TestCase.assertNotNull;
 import static junit.framework.TestCase.assertTrue;
 import static org.junit.Assert.assertNull;
-
 import java.io.IOException;
 import java.util.Arrays;
 import java.util.Collections;
@@ -29,41 +28,54 @@ import java.util.Random;
 import java.util.TreeMap;
 import java.util.concurrent.ThreadLocalRandom;
 
-import org.apache.hadoop.conf.Configuration;
-import org.apache.hadoop.hbase.HRegionInfo;
+import org.apache.hadoop.fs.FileSystem;
+import org.apache.hadoop.hbase.HBaseClassTestRule;
+import org.apache.hadoop.hbase.HBaseTestingUtility;
+import org.apache.hadoop.hbase.HConstants;
 import org.apache.hadoop.hbase.ServerName;
+import org.apache.hadoop.hbase.client.RegionInfo;
 import org.apache.hadoop.hbase.client.RegionReplicaUtil;
 import org.apache.hadoop.hbase.master.RackManager;
 import org.apache.hadoop.hbase.master.RegionPlan;
+import org.apache.hadoop.hbase.testclassification.MasterTests;
 import org.apache.hadoop.hbase.testclassification.MediumTests;
 import org.junit.BeforeClass;
+import org.junit.ClassRule;
 import org.junit.Test;
 import org.junit.experimental.categories.Category;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-@Category({MediumTests.class})
+@Category({ MasterTests.class, MediumTests.class })
 public class TestStochasticLoadBalancerHeterogeneousCost extends BalancerTestBase {
+  @ClassRule
+  public static final HBaseClassTestRule CLASS_RULE =
+      HBaseClassTestRule.forClass(TestStochasticLoadBalancerHeterogeneousCost.class);
 
   private static final Logger LOG =
       LoggerFactory.getLogger(TestStochasticLoadBalancerHeterogeneousCost.class);
-  private static final double allowedWindow = 1.20;
+  private static final double ALLOWED_WINDOW = 1.20;
+  private static final HBaseTestingUtility HTU = new HBaseTestingUtility();
+  private static String RULES_FILE;
 
   @BeforeClass
-  public static void beforeAllTests() {
-    conf = new Configuration();
-    conf.setFloat("hbase.master.balancer.stochastic.regionCountCost", 0);
-    conf.setFloat("hbase.master.balancer.stochastic.primaryRegionCountCost", 0);
-    conf.setFloat("hbase.master.balancer.stochastic.tableSkewCost", 0);
-    conf.setBoolean("hbase.master.balancer.stochastic.runMaxSteps", true);
-    conf.set(StochasticLoadBalancer.COST_FUNCTIONS_COST_FUNCTIONS_KEY,
+  public static void beforeAllTests() throws IOException {
+    BalancerTestBase.conf = HTU.getConfiguration();
+    BalancerTestBase.conf.setFloat("hbase.master.balancer.stochastic.regionCountCost", 0);
+    BalancerTestBase.conf.setFloat("hbase.master.balancer.stochastic.primaryRegionCountCost", 0);
+    BalancerTestBase.conf.setFloat("hbase.master.balancer.stochastic.tableSkewCost", 0);
+    BalancerTestBase.conf.set(StochasticLoadBalancer.COST_FUNCTIONS_COST_FUNCTIONS_KEY,
       HeterogeneousRegionCountCostFunction.class.getName());
-
-    conf.set(
+    // Need to ensure test dir has been created.
+    assertTrue(FileSystem.get(HTU.getConfiguration()).mkdirs(HTU.getDataTestDir()));
+    RULES_FILE = HTU.getDataTestDir(
+      TestStochasticLoadBalancerHeterogeneousCostRules.DEFAULT_RULES_FILE_NAME).toString();
+    BalancerTestBase.conf.set(
       HeterogeneousRegionCountCostFunction.HBASE_MASTER_BALANCER_HETEROGENEOUS_RULES_FILE,
-      TestStochasticLoadBalancerHeterogeneousCostRules.DEFAULT_RULES_TMP_LOCATION);
-    loadBalancer = new StochasticLoadBalancer();
-    loadBalancer.setConf(conf);
+      RULES_FILE);
+    BalancerTestBase.loadBalancer = new StochasticLoadBalancer();
+    BalancerTestBase.loadBalancer.setConf(BalancerTestBase.conf);
+    BalancerTestBase.loadBalancer.getCandidateGenerators().add(new FairRandomCandidateGenerator());
   }
 
   @Test
@@ -134,10 +146,11 @@ public class TestStochasticLoadBalancerHeterogeneousCost extends BalancerTestBas
     final int numRegions = 120;
     final int numRegionsPerServer = 60;
 
-    TestStochasticLoadBalancerHeterogeneousCostRules.createSimpleRulesFile(rules);
-    final Map<ServerName, List<HRegionInfo>> serverMap =
+    TestStochasticLoadBalancerHeterogeneousCostRules.createRulesFile(RULES_FILE);
+    final Map<ServerName, List<RegionInfo>> serverMap =
         this.createServerMap(numNodes, numRegions, numRegionsPerServer, 1, 1);
-    final List<RegionPlan> plans = BalancerTestBase.loadBalancer.balanceCluster(serverMap);
+    final List<RegionPlan> plans =
+        BalancerTestBase.loadBalancer.balanceTable(HConstants.ENSEMBLE_TABLE_NAME, serverMap);
     // As we disabled all the other cost functions, balancing only according to
     // the heterogeneous cost function should return nothing.
     assertNull(plans);
@@ -146,13 +159,13 @@ public class TestStochasticLoadBalancerHeterogeneousCost extends BalancerTestBas
   private void testHeterogeneousWithCluster(final int numNodes, final int numRegions,
       final int numRegionsPerServer, final List<String> rules) throws IOException {
 
-    TestStochasticLoadBalancerHeterogeneousCostRules.createSimpleRulesFile(rules);
-    final Map<ServerName, List<HRegionInfo>> serverMap =
+    TestStochasticLoadBalancerHeterogeneousCostRules.createRulesFile(RULES_FILE, rules);
+    final Map<ServerName, List<RegionInfo>> serverMap =
         this.createServerMap(numNodes, numRegions, numRegionsPerServer, 1, 1);
     this.testWithCluster(serverMap, null, true, false);
   }
 
-  protected void testWithCluster(final Map<ServerName, List<HRegionInfo>> serverMap,
+  protected void testWithCluster(final Map<ServerName, List<RegionInfo>> serverMap,
       final RackManager rackManager, final boolean assertFullyBalanced,
       final boolean assertFullyBalancedForReplicas) {
     final List<ServerAndLoad> list = this.convertToList(serverMap);
@@ -161,7 +174,8 @@ public class TestStochasticLoadBalancerHeterogeneousCost extends BalancerTestBas
     BalancerTestBase.loadBalancer.setRackManager(rackManager);
 
     // Run the balancer.
-    final List<RegionPlan> plans = BalancerTestBase.loadBalancer.balanceCluster(serverMap);
+    final List<RegionPlan> plans =
+        BalancerTestBase.loadBalancer.balanceTable(HConstants.ENSEMBLE_TABLE_NAME, serverMap);
     assertNotNull(plans);
 
     // Check to see that this actually got to a stable place.
@@ -174,7 +188,7 @@ public class TestStochasticLoadBalancerHeterogeneousCost extends BalancerTestBas
 
       if (assertFullyBalanced) {
         final List<RegionPlan> secondPlans =
-            BalancerTestBase.loadBalancer.balanceCluster(serverMap);
+            BalancerTestBase.loadBalancer.balanceTable(HConstants.ENSEMBLE_TABLE_NAME, serverMap);
         assertNull(secondPlans);
 
         // create external cost function to retrieve limit
@@ -199,8 +213,9 @@ public class TestStochasticLoadBalancerHeterogeneousCost extends BalancerTestBas
           // as the balancer is stochastic, we cannot check exactly the result of the balancing,
           // hence the allowedWindow parameter
           assertTrue("Host " + sn.getHostname() + " should be below "
-              + cf.overallUsage * allowedWindow * 100 + "%",
-            usage <= cf.overallUsage * allowedWindow);
+              + cf.overallUsage * ALLOWED_WINDOW * 100 + "%; " + cf.overallUsage +
+              ", " + usage + ", " + numberRegions + ", " + limit,
+            usage <= cf.overallUsage * ALLOWED_WINDOW);
         }
       }
 
@@ -211,7 +226,7 @@ public class TestStochasticLoadBalancerHeterogeneousCost extends BalancerTestBas
   }
 
   @Override
-  protected Map<ServerName, List<HRegionInfo>> createServerMap(int numNodes, int numRegions,
+  protected Map<ServerName, List<RegionInfo>> createServerMap(int numNodes, int numRegions,
       int numRegionsPerServer, int replication, int numTables) {
     // construct a cluster of numNodes, having a total of numRegions. Each RS will hold
     // numRegionsPerServer many regions except for the last one, which will host all the
@@ -221,10 +236,10 @@ public class TestStochasticLoadBalancerHeterogeneousCost extends BalancerTestBas
       cluster[i] = numRegionsPerServer;
     }
     cluster[cluster.length - 1] = numRegions - ((cluster.length - 1) * numRegionsPerServer);
-    Map<ServerName, List<HRegionInfo>> clusterState = mockClusterServers(cluster, numTables);
+    Map<ServerName, List<RegionInfo>> clusterState = mockClusterServers(cluster, numTables);
     if (replication > 0) {
       // replicate the regions to the same servers
-      for (List<HRegionInfo> regions : clusterState.values()) {
+      for (List<RegionInfo> regions : clusterState.values()) {
         int length = regions.size();
         for (int i = 0; i < length; i++) {
           for (int r = 1; r < replication; r++) {
@@ -238,14 +253,14 @@ public class TestStochasticLoadBalancerHeterogeneousCost extends BalancerTestBas
   }
 
   @Override
-  protected TreeMap<ServerName, List<HRegionInfo>> mockClusterServers(int[] mockCluster,
+  protected TreeMap<ServerName, List<RegionInfo>> mockClusterServers(int[] mockCluster,
       int numTables) {
     int numServers = mockCluster.length;
-    TreeMap<ServerName, List<HRegionInfo>> servers = new TreeMap<>();
+    TreeMap<ServerName, List<RegionInfo>> servers = new TreeMap<>();
     for (int i = 0; i < numServers; i++) {
       int numRegions = mockCluster[i];
       ServerAndLoad sal = createServer("rs" + i);
-      List<HRegionInfo> regions = randomRegions(numRegions, numTables);
+      List<RegionInfo> regions = randomRegions(numRegions, numTables);
       servers.put(sal.getServerName(), regions);
     }
     return servers;
@@ -264,4 +279,26 @@ public class TestStochasticLoadBalancerHeterogeneousCost extends BalancerTestBas
     ServerName sn = ServerName.valueOf(host, port, startCode);
     return new ServerAndLoad(sn, 0);
   }
+
+  static class FairRandomCandidateGenerator extends
+    StochasticLoadBalancer.RandomCandidateGenerator {
+
+    @Override
+    public BaseLoadBalancer.Cluster.Action pickRandomRegions(BaseLoadBalancer.Cluster cluster,
+      int thisServer, int otherServer) {
+      if (thisServer < 0 || otherServer < 0) {
+        return BaseLoadBalancer.Cluster.NullAction;
+      }
+
+      int thisRegion = pickRandomRegion(cluster, thisServer, 0.5);
+      int otherRegion = pickRandomRegion(cluster, otherServer, 0.5);
+
+      return getAction(thisServer, thisRegion, otherServer, otherRegion);
+    }
+
+    @Override
+    BaseLoadBalancer.Cluster.Action generate(BaseLoadBalancer.Cluster cluster) {
+      return super.generate(cluster);
+    }
+  }
 }
