@@ -17,26 +17,31 @@
  */
 package org.apache.hadoop.hbase.regionserver;
 
-import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNotNull;
-import static org.junit.Assert.assertTrue;
 
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.hbase.CompatibilityFactory;
-import org.apache.hadoop.hbase.TableName;
-import org.apache.hadoop.hbase.testclassification.SmallTests;
+import org.apache.hadoop.hbase.HBaseClassTestRule;
 import org.apache.hadoop.hbase.test.MetricsAssertHelper;
+import org.apache.hadoop.hbase.testclassification.RegionServerTests;
+import org.apache.hadoop.hbase.testclassification.SmallTests;
 import org.apache.hadoop.hbase.util.JvmPauseMonitor;
 import org.junit.Before;
 import org.junit.BeforeClass;
+import org.junit.ClassRule;
 import org.junit.Test;
 import org.junit.experimental.categories.Category;
 
 /**
  * Unit test version of rs metrics tests.
  */
-@Category(SmallTests.class)
+@Category({RegionServerTests.class, SmallTests.class})
 public class TestMetricsRegionServer {
+
+  @ClassRule
+  public static final HBaseClassTestRule CLASS_RULE =
+      HBaseClassTestRule.forClass(TestMetricsRegionServer.class);
+
   public static MetricsAssertHelper HELPER =
       CompatibilityFactory.getInstance(MetricsAssertHelper.class);
 
@@ -52,9 +57,7 @@ public class TestMetricsRegionServer {
   @Before
   public void setUp() {
     wrapper = new MetricsRegionServerWrapperStub();
-    Configuration conf = new Configuration(false);
-    conf.setBoolean(MetricsRegionServer.RS_ENABLE_SERVER_QUERY_METER_METRICS_KEY, false);
-    rsm = new MetricsRegionServer(wrapper, conf);
+    rsm = new MetricsRegionServer(wrapper, new Configuration(false), null);
     serverSource = rsm.getMetricsSource();
   }
 
@@ -81,6 +84,7 @@ public class TestMetricsRegionServer {
           + HELPER.getCounter("writeRequestCount", serverSource),
       serverSource);
     HELPER.assertCounter("readRequestCount", 997, serverSource);
+    HELPER.assertCounter("filteredReadRequestCount", 1997, serverSource);
     HELPER.assertCounter("writeRequestCount", 707, serverSource);
     HELPER.assertCounter("checkMutateFailedCount", 401, serverSource);
     HELPER.assertCounter("checkMutatePassedCount", 405, serverSource);
@@ -145,6 +149,8 @@ public class TestMetricsRegionServer {
     for (int i=0; i < 17; i ++) {
       rsm.updatePut(null, 17);
       rsm.updateDelete(null, 17);
+      rsm.updatePut(null, 1006);
+      rsm.updateDelete(null, 1003);
       rsm.updateCheckAndDelete(null, 17);
       rsm.updateCheckAndPut(null, 17);
     }
@@ -154,29 +160,29 @@ public class TestMetricsRegionServer {
     HELPER.assertCounter("getNumOps", 28, serverSource);
     HELPER.assertCounter("incrementNumOps", 30, serverSource);
     HELPER.assertCounter("putBatchNumOps", 32, serverSource);
-    HELPER.assertCounter("putNumOps", 17, serverSource);
-    HELPER.assertCounter("deleteNumOps", 17, serverSource);
+    HELPER.assertCounter("putNumOps", 34, serverSource);
+    HELPER.assertCounter("deleteNumOps", 34, serverSource);
     HELPER.assertCounter("checkAndDeleteNumOps", 17, serverSource);
     HELPER.assertCounter("checkAndPutNumOps", 17, serverSource);
 
 
     HELPER.assertCounter("slowAppendCount", 12, serverSource);
-    HELPER.assertCounter("slowDeleteCount", 13, serverSource);
+    HELPER.assertCounter("slowDeleteCount", 17, serverSource);
     HELPER.assertCounter("slowGetCount", 14, serverSource);
     HELPER.assertCounter("slowIncrementCount", 15, serverSource);
-    HELPER.assertCounter("slowPutCount", 16, serverSource);
+    HELPER.assertCounter("slowPutCount", 17, serverSource);
   }
 
   @Test
   public void testFlush() {
-    rsm.updateFlush(1, 2, 3);
+    rsm.updateFlush(null, 1, 2, 3);
     HELPER.assertCounter("flushTime_num_ops", 1, serverSource);
     HELPER.assertCounter("flushMemstoreSize_num_ops", 1, serverSource);
     HELPER.assertCounter("flushOutputSize_num_ops", 1, serverSource);
     HELPER.assertCounter("flushedMemstoreBytes", 2, serverSource);
     HELPER.assertCounter("flushedOutputBytes", 3, serverSource);
 
-    rsm.updateFlush(10, 20, 30);
+    rsm.updateFlush(null, 10, 20, 30);
     HELPER.assertCounter("flushTimeNumOps", 2, serverSource);
     HELPER.assertCounter("flushMemstoreSize_num_ops", 2, serverSource);
     HELPER.assertCounter("flushOutputSize_num_ops", 2, serverSource);
@@ -186,7 +192,7 @@ public class TestMetricsRegionServer {
 
   @Test
   public void testCompaction() {
-    rsm.updateCompaction(false, 1, 2, 3, 4, 5);
+    rsm.updateCompaction(null, false, 1, 2, 3, 4, 5);
     HELPER.assertCounter("compactionTime_num_ops", 1, serverSource);
     HELPER.assertCounter("compactionInputFileCount_num_ops", 1, serverSource);
     HELPER.assertCounter("compactionInputSize_num_ops", 1, serverSource);
@@ -194,7 +200,7 @@ public class TestMetricsRegionServer {
     HELPER.assertCounter("compactedInputBytes", 4, serverSource);
     HELPER.assertCounter("compactedoutputBytes", 5, serverSource);
 
-    rsm.updateCompaction(false, 10, 20, 30, 40, 50);
+    rsm.updateCompaction(null, false, 10, 20, 30, 40, 50);
     HELPER.assertCounter("compactionTime_num_ops", 2, serverSource);
     HELPER.assertCounter("compactionInputFileCount_num_ops", 2, serverSource);
     HELPER.assertCounter("compactionInputSize_num_ops", 2, serverSource);
@@ -203,7 +209,7 @@ public class TestMetricsRegionServer {
     HELPER.assertCounter("compactedoutputBytes", 55, serverSource);
 
     // do major compaction
-    rsm.updateCompaction(true, 100, 200, 300, 400, 500);
+    rsm.updateCompaction(null, true, 100, 200, 300, 400, 500);
 
     HELPER.assertCounter("compactionTime_num_ops", 3, serverSource);
     HELPER.assertCounter("compactionInputFileCount_num_ops", 3, serverSource);
@@ -238,26 +244,5 @@ public class TestMetricsRegionServer {
     HELPER.assertCounter("pauseTimeWithGc_num_ops", 1, serverSource);
   }
 
-  @Test
-  public void testServerQueryMeterSwitch() {
-    TableName tn1 = TableName.valueOf("table1");
-    // has been set disable in setUp()
-    rsm.updateReadQueryMeter(tn1, 500L);
-    assertFalse(HELPER.checkGaugeExists("ServerReadQueryPerSecond_count", serverSource));
-    rsm.updateWriteQueryMeter(tn1, 500L);
-    assertFalse(HELPER.checkGaugeExists("ServerWriteQueryPerSecond_count", serverSource));
-
-    // enable
-    Configuration conf = new Configuration(false);
-    conf.setBoolean(MetricsRegionServer.RS_ENABLE_SERVER_QUERY_METER_METRICS_KEY, true);
-    rsm = new MetricsRegionServer(wrapper, conf);
-    serverSource = rsm.getMetricsSource();
-    rsm.updateReadQueryMeter(tn1, 500L);
-    assertTrue(HELPER.checkGaugeExists("ServerWriteQueryPerSecond_count", serverSource));
-    HELPER.assertGauge("ServerReadQueryPerSecond_count", 500L, serverSource);
-    assertTrue(HELPER.checkGaugeExists("ServerWriteQueryPerSecond_count", serverSource));
-    rsm.updateWriteQueryMeter(tn1, 500L);
-    HELPER.assertGauge("ServerWriteQueryPerSecond_count", 500L, serverSource);
-  }
 }
 
