@@ -17,7 +17,7 @@
 # limitations under the License.
 #
 
-require 'hbase'
+require 'hbase_constants'
 
 include HBaseConstants
 
@@ -104,7 +104,7 @@ module Hbase
       @test_name = "hbase_shell_tests_table"
       create_test_table(@test_name)
       @test_table = table(@test_name)
-
+      
       # Insert data to perform delete operations
       @test_table.put("102", "x:a", "2", 1212)
       @test_table.put(103, "x:a", "3", 1214)
@@ -114,6 +114,13 @@ module Hbase
 
       @test_table.put(105, "x:a", "3")
       @test_table.put(105, "x:a", "4")
+
+      @test_table.put(106, "x:a", "3", 1588765900000)
+      @test_table.put(106, "x:b", "4", 1588765900010)
+
+      @test_table.put("111", "x:a", "5")
+      @test_table.put("111", "x:b", "6")
+      @test_table.put("112", "x:a", "5")
     end
 
     def teardown
@@ -142,7 +149,6 @@ module Hbase
     end
 
     #-------------------------------------------------------------------------------
-
     define_test "delete should work with string keys" do
       @test_table.delete('102', 'x:a', 1212)
       res = @test_table._get_internal('102', 'x:a')
@@ -165,17 +171,45 @@ module Hbase
       assert_nil(res)
     end
 
+    define_test "deleteall should work with timestamps but w/o columns" do
+      @test_table.deleteall("106", "", 1588765900005)
+      res = @test_table._get_internal('106', 'x:a')
+      assert_nil(res)
+      res = @test_table._get_internal('106', 'x:b')
+      assert_not_nil(res)
+    end
+
     define_test "deleteall should work with integer keys" do
       @test_table.deleteall(105)
       res = @test_table._get_internal('105', 'x:a')
       assert_nil(res)
     end
 
+    define_test "deletall should work with row prefix" do
+      @test_table.deleteall({ROWPREFIXFILTER => '11'})
+      res1 = @test_table._get_internal('111')
+      assert_nil(res1)
+      res2 = @test_table._get_internal('112')
+      assert_nil(res2)
+    end
+
     define_test "append should work with value" do
       @test_table.append("123", 'x:cnt2', '123')
       assert_equal("123123", @test_table._append_internal("123", 'x:cnt2', '123'))
     end
+
+    define_test 'append should work without qualifier' do
+      @test_table.append('1001', 'x', '123')
+      assert_equal('123321', @test_table._append_internal('1001', 'x', '321'))
+    end
+
     #-------------------------------------------------------------------------------
+    define_test 'incr should work without qualifier' do
+      @test_table.incr('1010', 'x', 123)
+      assert_equal(123, @test_table._get_counter_internal('1010', 'x'))
+      @test_table.incr('1010', 'x', 123)
+      assert_equal(246, @test_table._get_counter_internal('1010', 'x'))
+    end
 
     define_test "get_counter should work with integer keys" do
       @test_table.incr(12345, 'x:cnt')
@@ -229,6 +263,31 @@ module Hbase
       assert(!rows.empty?)
     end
 
+    define_test "count should support STARTROW parameter" do
+      count = @test_table.count STARTROW => '4'
+      assert(count == 0)
+    end
+
+    define_test "count should support STOPROW parameter" do
+      count = @test_table.count STOPROW => '0'
+      assert(count == 0)
+    end
+
+    define_test "count should support COLUMNS parameter" do
+      @test_table.put(4, "x:c", "31")
+      begin
+        count = @test_table.count COLUMNS => [ 'x:c']
+        assert(count == 1)
+      ensure
+        @test_table.deleteall(4, 'x:c')
+      end
+    end
+
+    define_test "count should support FILTER parameter" do
+      count = @test_table.count FILTER => "ValueFilter(=, 'binary:11')"
+      assert(count == 1)
+    end
+
     #-------------------------------------------------------------------------------
 
     define_test "get should work w/o columns specification" do
@@ -302,7 +361,7 @@ module Hbase
       assert_nil(res['x:a'])
       assert_not_nil(res['x:b'])
     end
-    
+
     define_test 'get should work with hash columns spec and TIMESTAMP and' \
                 ' AUTHORIZATIONS' do
       res = @test_table._get_internal('1', TIMESTAMP => 1234, AUTHORIZATIONS=>['PRIVATE'])
@@ -389,6 +448,21 @@ module Hbase
       end
     end
 
+    define_test 'get should work with a custom converter class' do
+      @test_table.put(1, 'x:v', 1234)
+      begin
+        res = @test_table._get_internal('1', 'COLUMNS' =>
+                    ['x:v:c(org.apache.hadoop.hbase.util.Bytes).len'])
+        assert_not_nil(res)
+        assert_kind_of(Hash, res)
+        assert_not_nil(res['x:v'])
+        assert_not_nil(/value=4/.match(res['x:v']))
+      ensure
+        # clean up newly added columns for this test only.
+        @test_table.deleteall(1, 'x:v')
+      end
+    end
+
     #-------------------------------------------------------------------------------
 
     define_test "scan should work w/o any params" do
@@ -413,6 +487,16 @@ module Hbase
       assert_not_nil(res['2']['x:b'])
     end
 
+    define_test "scan should support STARTKEY parameter" do
+      res = @test_table._scan_internal STARTKEY => '2'
+      assert_not_nil(res)
+      assert_kind_of(Hash, res)
+      assert_nil(res['1'])
+      assert_not_nil(res['2'])
+      assert_not_nil(res['2']['x:a'])
+      assert_not_nil(res['2']['x:b'])
+    end
+
     define_test "scan should support STOPROW parameter" do
       res = @test_table._scan_internal STOPROW => '2'
       assert_not_nil(res)
@@ -423,7 +507,27 @@ module Hbase
       assert_nil(res['2'])
     end
 
-    define_test "scan should support ROWPREFIXFILTER parameter (test 1)" do
+    define_test "scan should support ENDROW parameter" do
+      res = @test_table._scan_internal ENDROW => '2'
+      assert_not_nil(res)
+      assert_kind_of(Hash, res)
+      assert_not_nil(res['1'])
+      assert_not_nil(res['1']['x:a'])
+      assert_not_nil(res['1']['x:b'])
+      assert_nil(res['2'])
+    end
+
+    define_test "scan should support ENDKEY parameter" do
+      res = @test_table._scan_internal ENDKEY => '2'
+      assert_not_nil(res)
+      assert_kind_of(Hash, res)
+      assert_not_nil(res['1'])
+      assert_not_nil(res['1']['x:a'])
+      assert_not_nil(res['1']['x:b'])
+      assert_nil(res['2'])
+    end
+
+    define_test 'scan should support ROWPREFIXFILTER parameter (test 1)' do
       res = @test_table._scan_internal ROWPREFIXFILTER => '1'
       assert_not_nil(res)
       assert_kind_of(Hash, res)
@@ -433,7 +537,7 @@ module Hbase
       assert_nil(res['2'])
     end
 
-    define_test "scan should support ROWPREFIXFILTER parameter (test 2)" do
+    define_test 'scan should support ROWPREFIXFILTER parameter (test 2)' do
       res = @test_table._scan_internal ROWPREFIXFILTER => '2'
       assert_not_nil(res)
       assert_kind_of(Hash, res)
@@ -443,7 +547,7 @@ module Hbase
       assert_not_nil(res['2']['x:b'])
     end
 
-    define_test "scan should support LIMIT parameter" do
+    define_test 'scan should support LIMIT parameter' do
       res = @test_table._scan_internal LIMIT => 1
       assert_not_nil(res)
       assert_kind_of(Hash, res)
@@ -509,6 +613,30 @@ module Hbase
       assert_nil(res['2']['x:b'])
     end
 
+    define_test 'scan should support REGION_REPLICA_ID' do
+      res = @test_table._scan_internal REGION_REPLICA_ID => 0
+      assert_not_nil(res)
+    end
+
+    define_test 'scan should support ISOLATION_LEVEL' do
+      res = @test_table._scan_internal ISOLATION_LEVEL => 'READ_COMMITTED'
+      assert_not_nil(res)
+    end
+
+    define_test 'scan should support READ_TYPE parameter' do
+      res = @test_table._scan_internal READ_TYPE => 'PREAD'
+      assert_not_nil(res)
+      res = @test_table._scan_internal READ_TYPE => 'STREAM'
+      assert_not_nil(res)
+      res = @test_table._scan_internal READ_TYPE => 'DEFAULT'
+      assert_not_nil(res)
+    end
+
+    define_test 'scan should support ALLOW_PARTIAL_RESULTS' do
+      res = @test_table._scan_internal ALLOW_PARTIAL_RESULTS => true
+      assert_not_nil(res)
+    end
+
     define_test "scan should work with raw and version parameter" do
       # Create test table if it does not exist
       @test_name_raw = "hbase_shell_tests_raw_scan"
@@ -523,22 +651,41 @@ module Hbase
       @test_table.put(2, "x:raw1", 11)
 
       args = {}
-      numRows = 0
-      count = @test_table._scan_internal(args) do |row, cells| # Normal Scan
-        numRows += 1
+      num_rows = 0
+      @test_table._scan_internal(args) do # Normal Scan
+        num_rows += 1
       end
-      assert_equal(numRows, 2, "Num rows scanned without RAW/VERSIONS are not 2") 
+      assert_equal(num_rows, 2,
+                   'Num rows scanned without RAW/VERSIONS are not 2')
 
-      args = {VERSIONS=>10,RAW=>true} # Since 4 versions of row with rowkey 2 is been added, we can use any number >= 4 for VERSIONS to scan all 4 versions.
-      numRows = 0
-      count = @test_table._scan_internal(args) do |row, cells| # Raw Scan
-        numRows += 1
+      args = { VERSIONS => 10, RAW => true } # Since 4 versions of row with rowkey 2 is been added, we can use any number >= 4 for VERSIONS to scan all 4 versions.
+      num_rows = 0
+      @test_table._scan_internal(args) do # Raw Scan
+        num_rows += 1
       end
-      assert_equal(numRows, 5, "Num rows scanned without RAW/VERSIONS are not 5") # 5 since , 1 from row key '1' and other 4 from row key '4'
+      # 5 since , 1 from row key '1' and other 4 from row key '4'
+      assert_equal(num_rows, 5,
+                   'Num rows scanned without RAW/VERSIONS are not 5')
+
+      @test_table.delete(1, 'x:a')
+      args = {}
+      num_rows = 0
+      @test_table._scan_internal(args) do # Normal Scan
+        num_rows += 1
+      end
+      assert_equal(num_rows, 1,
+                   'Num rows scanned without RAW/VERSIONS are not 1')
+
+      args = { VERSIONS => 10, RAW => true }
+      num_rows = 0
+      @test_table._scan_internal(args) do # Raw Scan
+        num_rows += 1
+      end
+      # 6 since , 2 from row key '1' and other 4 from row key '4'
+      assert_equal(num_rows, 6,
+                   'Num rows scanned without RAW/VERSIONS are not 5')
     end
 
-
-
     define_test "scan should fail on invalid COLUMNS parameter types" do
       assert_raise(ArgumentError) do
         @test_table._scan_internal COLUMNS => {}
@@ -607,6 +754,26 @@ module Hbase
       end
     end
 
+    define_test "scan hbase meta table" do
+      res = table("hbase:meta")._scan_internal
+      assert_not_nil(res)
+    end
+
+    define_test 'scan should work with a custom converter class' do
+      @test_table.put(1, 'x:v', 1234)
+      begin
+        res = @test_table._scan_internal 'COLUMNS' =>
+                    ['x:v:c(org.apache.hadoop.hbase.util.Bytes).len']
+        assert_not_nil(res)
+        assert_kind_of(Hash, res)
+        assert_not_nil(res['1']['x:v'])
+        assert_not_nil(/value=4/.match(res['1']['x:v']))
+      ensure
+        # clean up newly added columns for this test only.
+        @test_table.deleteall(1, 'x:v')
+      end
+    end
+
     define_test "mutation with TTL should expire" do
       @test_table.put('ttlTest', 'x:a', 'foo', { TTL => 1000 } )
       begin
@@ -620,20 +787,38 @@ module Hbase
       end
     end
 
-    define_test "Split count for a table" do
-      @testTableName = "tableWithSplits"
-      create_test_table_with_splits(@testTableName, SPLITS => ['10', '20', '30', '40'])
-      @table = table(@testTableName)
-      splits = @table._get_splits_internal()
-      #Total splits is 5 but here count is 4 as we ignore implicit empty split.
+    define_test 'Split count for a table' do
+      @test_table_name = 'table_with_splits'
+      create_test_table_with_splits(@test_table_name, SPLITS => %w[10 20 30 40])
+      @table = table(@test_table_name)
+      splits = @table._get_splits_internal
+      # Total splits is 5 but here count is 4 as we ignore implicit empty split.
       assert_equal(4, splits.size)
-      assert_equal(["10", "20", "30", "40"], splits)
-      drop_test_table(@testTableName)
+      assert_equal(%w[10 20 30 40], splits)
+      drop_test_table(@test_table_name)
     end
 
-    define_test "Split count for a empty table" do
-      splits = @test_table._get_splits_internal()
-      #Empty split should not be part of this array.
+    define_test 'Split count for a table by file' do
+      @test_table_name = 'table_with_splits_file'
+      @splits_file = 'target/generated-test-sources/splits.txt'
+      File.exist?(@splits_file) && File.delete(@splits_file)
+      file = File.new(@splits_file, 'w')
+      %w[10 20 30 40].each { |item| file.puts item }
+      file.close
+      create_test_table_with_splits_file(@test_table_name,
+                                         SPLITS_FILE => @splits_file)
+      @table = table(@test_table_name)
+      splits = @table._get_splits_internal
+      # Total splits is 5 but here count is 4 as we ignore implicit empty split.
+      assert_equal(4, splits.size)
+      assert_equal(%w[10 20 30 40], splits)
+      drop_test_table(@test_table_name)
+      File.delete(@splits_file)
+    end
+
+    define_test 'Split count for a empty table' do
+      splits = @test_table._get_splits_internal
+      # Empty split should not be part of this array.
       assert_equal(0, splits.size)
       assert_equal([], splits)
     end
@@ -650,6 +835,17 @@ module Hbase
       assert_equal(['10'], splits)
       drop_test_table(@test_table_name)
     end
+
+    define_test "scan should throw an exception on a disabled table" do
+      @test_table.disable
+      begin
+        assert_raise(RuntimeError) do
+          @test_table.scan
+        end
+      ensure
+        @test_table.enable
+      end
+    end
   end
   # rubocop:enable Metrics/ClassLength
 end
