@@ -1,5 +1,4 @@
 /*
- *
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -33,11 +32,10 @@ import java.util.Collection;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
-
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import java.util.stream.Collectors;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.hbase.CompatibilityFactory;
+import org.apache.hadoop.hbase.HBaseClassTestRule;
 import org.apache.hadoop.hbase.HBaseTestingUtility;
 import org.apache.hadoop.hbase.HColumnDescriptor;
 import org.apache.hadoop.hbase.HConstants;
@@ -65,18 +63,21 @@ import org.apache.hadoop.hbase.thrift.generated.TRowResult;
 import org.apache.hadoop.hbase.thrift.generated.TScan;
 import org.apache.hadoop.hbase.thrift.generated.TThriftServerType;
 import org.apache.hadoop.hbase.util.Bytes;
+import org.apache.hadoop.hbase.util.TableDescriptorChecker;
 import org.apache.hadoop.hbase.util.Threads;
-import org.apache.thrift.TException;
 import org.apache.thrift.protocol.TBinaryProtocol;
 import org.apache.thrift.protocol.TProtocol;
 import org.apache.thrift.transport.TSocket;
 import org.apache.thrift.transport.TTransport;
 import org.junit.AfterClass;
 import org.junit.BeforeClass;
+import org.junit.ClassRule;
 import org.junit.Rule;
 import org.junit.Test;
 import org.junit.experimental.categories.Category;
 import org.junit.rules.TestName;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 /**
  * Unit testing for ThriftServerRunner.HBaseServiceHandler, a part of the
@@ -85,8 +86,12 @@ import org.junit.rules.TestName;
 @Category({ClientTests.class, LargeTests.class})
 public class TestThriftServer {
 
+  @ClassRule
+  public static final HBaseClassTestRule CLASS_RULE =
+      HBaseClassTestRule.forClass(TestThriftServer.class);
+
   private static final HBaseTestingUtility UTIL = new HBaseTestingUtility();
-  private static final Log LOG = LogFactory.getLog(TestThriftServer.class);
+  private static final Logger LOG = LoggerFactory.getLogger(TestThriftServer.class);
   private static final MetricsAssertHelper metricsHelper = CompatibilityFactory
       .getInstance(MetricsAssertHelper.class);
   protected static final int MAXVERSIONS = 3;
@@ -118,6 +123,7 @@ public class TestThriftServer {
   @BeforeClass
   public static void beforeClass() throws Exception {
     UTIL.getConfiguration().setBoolean(COALESCE_INC_KEY, true);
+    UTIL.getConfiguration().setBoolean(TableDescriptorChecker.TABLE_SANITY_CHECKS, false);
     UTIL.getConfiguration().setInt("hbase.client.retries.number", 3);
     UTIL.startMiniCluster();
   }
@@ -160,8 +166,8 @@ public class TestThriftServer {
    */
   public void doTestTableCreateDrop() throws Exception {
     ThriftHBaseServiceHandler handler =
-        new ThriftHBaseServiceHandler(UTIL.getConfiguration(),
-            UserProvider.instantiate(UTIL.getConfiguration()));
+      new ThriftHBaseServiceHandler(UTIL.getConfiguration(),
+        UserProvider.instantiate(UTIL.getConfiguration()));
     doTestTableCreateDrop(handler);
   }
 
@@ -183,10 +189,6 @@ public class TestThriftServer {
       Threads.sleepWithoutInterrupt(3000);
       return super.getTableNames();
     }
-
-    @Override public TThriftServerType getThriftServerType() throws TException {
-      return null;
-    }
   }
 
   /**
@@ -224,22 +226,20 @@ public class TestThriftServer {
     createTestTables(handler);
     dropTestTables(handler);;
     metricsHelper.assertCounter("createTable_num_ops", currentCountCreateTable + 2,
-        metrics.getSource());
+      metrics.getSource());
     metricsHelper.assertCounter("deleteTable_num_ops", currentCountDeleteTable + 2,
-        metrics.getSource());
+      metrics.getSource());
     metricsHelper.assertCounter("disableTable_num_ops", currentCountDisableTable + 2,
-        metrics.getSource());
+      metrics.getSource());
     handler.getTableNames(); // This will have an artificial delay.
 
     // 3 to 6 seconds (to account for potential slowness), measured in nanoseconds
-    try {
-      metricsHelper.assertGaugeGt("getTableNames_avg_time",
-        3L * 1000 * 1000 * 1000, metrics.getSource());
-      metricsHelper.assertGaugeLt("getTableNames_avg_time",
-        6L * 1000 * 1000 * 1000, metrics.getSource());
-    } catch (AssertionError e) {
-      LOG.info("Fix me!  Why does this happen?  A concurrent cluster running?", e);
-    }
+   try {
+     metricsHelper.assertGaugeGt("getTableNames_avg_time", 3L * 1000 * 1000 * 1000, metrics.getSource());
+     metricsHelper.assertGaugeLt("getTableNames_avg_time",6L * 1000 * 1000 * 1000, metrics.getSource());
+   } catch (AssertionError e) {
+     LOG.info("Fix me!  Why does this happen?  A concurrent cluster running?", e);
+   }
   }
 
   private static Hbase.Iface getHandlerForMetricsTest(ThriftMetrics metrics, Configuration conf)
@@ -255,13 +255,15 @@ public class TestThriftServer {
 
   public static void createTestTables(Hbase.Iface handler) throws Exception {
     // Create/enable/disable/delete tables, ensure methods act correctly
-    assertEquals(handler.getTableNames().size(), 0);
+    List<java.nio.ByteBuffer> bbs = handler.getTableNames();
+    assertEquals(bbs.stream().map(b -> Bytes.toString(b.array())).
+      collect(Collectors.joining(",")), 0, bbs.size());
     handler.createTable(tableAname, getColumnDescriptors());
-    assertEquals(handler.getTableNames().size(), 1);
-    assertEquals(handler.getColumnDescriptors(tableAname).size(), 2);
+    assertEquals(1, handler.getTableNames().size());
+    assertEquals(2, handler.getColumnDescriptors(tableAname).size());
     assertTrue(handler.isTableEnabled(tableAname));
     handler.createTable(tableBname, getColumnDescriptors());
-    assertEquals(handler.getTableNames().size(), 2);
+    assertEquals(2, handler.getTableNames().size());
   }
 
   public static void checkTableList(Hbase.Iface handler) throws Exception {
@@ -286,8 +288,8 @@ public class TestThriftServer {
 
   public void doTestIncrements() throws Exception {
     ThriftHBaseServiceHandler handler =
-        new ThriftHBaseServiceHandler(UTIL.getConfiguration(),
-            UserProvider.instantiate(UTIL.getConfiguration()));
+      new ThriftHBaseServiceHandler(UTIL.getConfiguration(),
+        UserProvider.instantiate(UTIL.getConfiguration()));
     createTestTables(handler);
     doTestIncrements(handler);
     dropTestTables(handler);
@@ -334,8 +336,8 @@ public class TestThriftServer {
    */
   public void doTestTableMutations() throws Exception {
     ThriftHBaseServiceHandler handler =
-        new ThriftHBaseServiceHandler(UTIL.getConfiguration(),
-            UserProvider.instantiate(UTIL.getConfiguration()));
+      new ThriftHBaseServiceHandler(UTIL.getConfiguration(),
+        UserProvider.instantiate(UTIL.getConfiguration()));
     doTestTableMutations(handler);
   }
 
@@ -350,11 +352,11 @@ public class TestThriftServer {
 
     // Assert that the changes were made
     assertEquals(valueAname,
-        handler.get(tableAname, rowAname, columnAname, null).get(0).value);
+      handler.get(tableAname, rowAname, columnAname, null).get(0).value);
     TRowResult rowResult1 = handler.getRow(tableAname, rowAname, null).get(0);
     assertEquals(rowAname, rowResult1.row);
     assertEquals(valueBname,
-        rowResult1.columns.get(columnBname).value);
+      rowResult1.columns.get(columnBname).value);
 
     // Apply a few BatchMutations for rowA and rowB
     // rowAmutations.add(new Mutation(true, columnAname, null));
@@ -413,8 +415,8 @@ public class TestThriftServer {
   public void doTestTableTimestampsAndColumns() throws Exception {
     // Setup
     ThriftHBaseServiceHandler handler =
-        new ThriftHBaseServiceHandler(UTIL.getConfiguration(),
-            UserProvider.instantiate(UTIL.getConfiguration()));
+      new ThriftHBaseServiceHandler(UTIL.getConfiguration(),
+        UserProvider.instantiate(UTIL.getConfiguration()));
     handler.createTable(tableAname, getColumnDescriptors());
 
     // Apply timestamped Mutations to rowA
@@ -436,9 +438,9 @@ public class TestThriftServer {
 
     // Assert that the timestamp-related methods retrieve the correct data
     assertEquals(2, handler.getVerTs(tableAname, rowAname, columnBname, time2,
-        MAXVERSIONS, null).size());
+      MAXVERSIONS, null).size());
     assertEquals(1, handler.getVerTs(tableAname, rowAname, columnBname, time1,
-        MAXVERSIONS, null).size());
+      MAXVERSIONS, null).size());
 
     TRowResult rowResult1 = handler.getRowTs(tableAname, rowAname, time1, null).get(0);
     TRowResult rowResult2 = handler.getRowTs(tableAname, rowAname, time2, null).get(0);
@@ -493,8 +495,8 @@ public class TestThriftServer {
   public void doTestTableScanners() throws Exception {
     // Setup
     ThriftHBaseServiceHandler handler =
-        new ThriftHBaseServiceHandler(UTIL.getConfiguration(),
-            UserProvider.instantiate(UTIL.getConfiguration()));
+      new ThriftHBaseServiceHandler(UTIL.getConfiguration(),
+        UserProvider.instantiate(UTIL.getConfiguration()));
     handler.createTable(tableAname, getColumnDescriptors());
 
     // Apply timestamped Mutations to rowA
@@ -613,8 +615,8 @@ public class TestThriftServer {
    */
   public void doTestGetTableRegions() throws Exception {
     ThriftHBaseServiceHandler handler =
-        new ThriftHBaseServiceHandler(UTIL.getConfiguration(),
-            UserProvider.instantiate(UTIL.getConfiguration()));
+      new ThriftHBaseServiceHandler(UTIL.getConfiguration(),
+        UserProvider.instantiate(UTIL.getConfiguration()));
     doTestGetTableRegions(handler);
   }
 
@@ -626,13 +628,13 @@ public class TestThriftServer {
     List<TRegionInfo> regions = handler.getTableRegions(tableAname);
     int regionCount = regions.size();
     assertEquals("empty table should have only 1 region, " +
-        "but found " + regionCount, 1, regionCount);
+            "but found " + regionCount, 1, regionCount);
     LOG.info("Region found:" + regions.get(0));
     handler.disableTable(tableAname);
     handler.deleteTable(tableAname);
     regionCount = handler.getTableRegions(tableAname).size();
     assertEquals("non-existing table should have 0 region, " +
-        "but found " + regionCount, 0, regionCount);
+            "but found " + regionCount, 0, regionCount);
   }
 
   public void doTestFilterRegistration() throws Exception {
@@ -649,8 +651,8 @@ public class TestThriftServer {
 
   public void doTestGetRegionInfo() throws Exception {
     ThriftHBaseServiceHandler handler =
-        new ThriftHBaseServiceHandler(UTIL.getConfiguration(),
-            UserProvider.instantiate(UTIL.getConfiguration()));
+      new ThriftHBaseServiceHandler(UTIL.getConfiguration(),
+        UserProvider.instantiate(UTIL.getConfiguration()));
     doTestGetRegionInfo(handler);
   }
 
@@ -664,7 +666,7 @@ public class TestThriftServer {
           HConstants.NINES, false);
       TRegionInfo regionInfo = handler.getRegionInfo(ByteBuffer.wrap(searchRow));
       assertTrue(Bytes.toStringBinary(regionInfo.getName()).startsWith(
-          Bytes.toStringBinary(tableAname)));
+            Bytes.toStringBinary(tableAname)));
     } finally {
       handler.disableTable(tableAname);
       handler.deleteTable(tableAname);
@@ -678,8 +680,8 @@ public class TestThriftServer {
    */
   public static void doTestAppend() throws Exception {
     ThriftHBaseServiceHandler handler =
-        new ThriftHBaseServiceHandler(UTIL.getConfiguration(),
-            UserProvider.instantiate(UTIL.getConfiguration()));
+      new ThriftHBaseServiceHandler(UTIL.getConfiguration(),
+        UserProvider.instantiate(UTIL.getConfiguration()));
     handler.createTable(tableAname, getColumnDescriptors());
     try {
       List<Mutation> mutations = new ArrayList<>(1);
@@ -697,7 +699,7 @@ public class TestThriftServer {
       TRowResult rowResult = handler.getRow(tableAname, rowAname, null).get(0);
       assertEquals(rowAname, rowResult.row);
       assertArrayEquals(Bytes.add(valueAname.array(), valueBname.array()),
-          rowResult.columns.get(columnAname).value.array());
+        rowResult.columns.get(columnAname).value.array());
     } finally {
       handler.disableTable(tableAname);
       handler.deleteTable(tableAname);
@@ -712,8 +714,8 @@ public class TestThriftServer {
    */
   public static void doTestCheckAndPut() throws Exception {
     ThriftHBaseServiceHandler handler =
-        new ThriftHBaseServiceHandler(UTIL.getConfiguration(),
-            UserProvider.instantiate(UTIL.getConfiguration()));
+      new ThriftHBaseServiceHandler(UTIL.getConfiguration(),
+        UserProvider.instantiate(UTIL.getConfiguration()));
     handler.createTable(tableAname, getColumnDescriptors());
     try {
       List<Mutation> mutations = new ArrayList<>(1);
@@ -784,8 +786,8 @@ public class TestThriftServer {
   }
 
   private void testExceptionType(Hbase.Iface handler, ThriftMetrics metrics,
-      ByteBuffer tTableName, String rowkey,
-      ErrorThrowingGetObserver.ErrorType errorType) throws Exception {
+                                 ByteBuffer tTableName, String rowkey,
+                                 ErrorThrowingGetObserver.ErrorType errorType) throws Exception {
     long preGetCounter = metricsHelper.getCounter("getRow_num_ops", metrics.getSource());
     String exceptionKey = errorType.getMetricName();
     long preExceptionCounter = metricsHelper.checkCounterExists(exceptionKey, metrics.getSource()) ?
